@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -474,8 +474,7 @@ class DebugClientBase(object):
                     pass
                 sys.argv.append(fn)
                 sys.argv.extend(eval(args))
-                sys.path[0] = os.path.dirname(sys.argv[0])
-                sys.path.insert(0, '')
+                sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
                 else:
@@ -520,8 +519,7 @@ class DebugClientBase(object):
                     pass
                 sys.argv.append(fn)
                 sys.argv.extend(eval(args))
-                sys.path[0] = os.path.dirname(sys.argv[0])
-                sys.path.insert(0, '')
+                sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
                 else:
@@ -558,8 +556,7 @@ class DebugClientBase(object):
                     pass
                 sys.argv.append(fn)
                 sys.argv.extend(eval(args))
-                sys.path[0] = os.path.dirname(sys.argv[0])
-                sys.path.insert(0, '')
+                sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
                 else:
@@ -597,8 +594,7 @@ class DebugClientBase(object):
                     pass
                 sys.argv.append(fn)
                 sys.argv.extend(eval(args))
-                sys.path[0] = os.path.dirname(sys.argv[0])
-                sys.path.insert(0, '')
+                sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
                 else:
@@ -1074,12 +1070,20 @@ class DebugClientBase(object):
         if remoteAddress is None:                               # default: 127.0.0.1
             sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             sock.connect((DebugAddress, port))
-        elif ":" in remoteAddress:                              # IPv6
-            sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
-            sock.connect((remoteAddress, port))
-        else:                                                   # IPv4
-            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            sock.connect((remoteAddress, port))
+        else:
+            if "@@i" in remoteAddress:
+                remoteAddress, index = remoteAddress.split("@@i")
+            else:
+                index = 0
+            if ":" in remoteAddress:                              # IPv6
+                sockaddr = socket.getaddrinfo(
+                    remoteAddress, port, 0, 0, socket.SOL_TCP)[0][-1]
+                sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
+                sockaddr = sockaddr[:-1] + (int(index),)
+                sock.connect(sockaddr)
+            else:                                                   # IPv4
+                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                sock.connect((remoteAddress, port))
 
         self.readstream = AsyncFile(sock, sys.stdin.mode, sys.stdin.name)
         self.writestream = AsyncFile(sock, sys.stdout.mode, sys.stdout.name)
@@ -1397,21 +1401,37 @@ class DebugClientBase(object):
                 else:
                     vlist = []
             else:
-                # format the dictionary found
-                if dictkeys is None:
-                    dictkeys = dict.keys()
-                else:
-                    # treatment for sequences and dictionaries
+                qtVariable = False
+                if len(udict) > 0:
                     if access:
-                        exec "dict = dict%s" % access
+                        exec 'qvar = udict%s' % access
+                    # this has to be in line with VariablesViewer.indicators
+                    elif rvar and rvar[0][-2:] in ["[]", "()", "{}"]:
+                        exec 'qvar = udict["%s"][%s]' % (rvar[0][:-2], rvar[1])
                     else:
-                        dict = dict[dictkeys[0]]
-                    if isDict:
+                        qvar = udict[var[-1]]
+                    qvtype = ("%s" % type(qvar))[1:-1].split()[1][1:-1]
+                    if qvtype.startswith("PyQt4"):
+                        qtVariable = True
+                
+                if qtVariable:
+                    vlist = self.__formatQt4Variable(qvar, qvtype)
+                else:
+                    # format the dictionary found
+                    if dictkeys is None:
                         dictkeys = dict.keys()
                     else:
-                        dictkeys = range(len(dict))
-                vlist = self.__formatVariablesList(dictkeys, dict, scope, filter, 
-                                                 formatSequences)
+                        # treatment for sequences and dictionaries
+                        if access:
+                            exec "dict = dict%s" % access
+                        else:
+                            dict = dict[dictkeys[0]]
+                        if isDict:
+                            dictkeys = dict.keys()
+                        else:
+                            dictkeys = range(len(dict))
+                    vlist = self.__formatVariablesList(dictkeys, dict, scope, filter, 
+                                                     formatSequences)
             varlist.extend(vlist)
         
             if obj is not None and not formatSequences:
@@ -1615,7 +1635,11 @@ class DebugClientBase(object):
                         if valtype == "classobj":
                             if ConfigVarTypeStrings.index('instance') in filter:
                                 continue
-                        elif ConfigVarTypeStrings.index('other') in filter:
+                        elif valtype == "sip.methoddescriptor":
+                            if ConfigVarTypeStrings.index('instance method') in filter:
+                                continue
+                        elif not valtype.startswith("PySide") and \
+                             ConfigVarTypeStrings.index('other') in filter:
                             continue
                     
                 try:
@@ -1774,8 +1798,7 @@ class DebugClientBase(object):
         self.dircache = []
         sys.argv = progargs[:]
         sys.argv[0] = os.path.abspath(sys.argv[0])
-        sys.path[0] = os.path.dirname(sys.argv[0])
-        sys.path.insert(0, '')
+        sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
         if wd == '':
             os.chdir(sys.path[1])
         else:
@@ -1861,7 +1884,7 @@ class DebugClientBase(object):
                     del args[0]
                     del args[0]
                 elif args[0] == '-p':
-                    port = long(args[1])
+                    port = int(args[1])
                     del args[0]
                     del args[0]
                 elif args[0] == '-w':
@@ -1904,7 +1927,7 @@ class DebugClientBase(object):
             if sys.argv[1] == '':
                 del sys.argv[1]
             try:
-                port = long(sys.argv[1])
+                port = int(sys.argv[1])
             except (ValueError, IndexError):
                 port = -1
             try:
@@ -1971,3 +1994,19 @@ class DebugClientBase(object):
             return
         
         DebugClientOrigClose(fd)
+        
+    def __getSysPath(self, firstEntry):
+        """
+        Private slot to calculate a path list including the PYTHONPATH 
+        environment variable.
+        
+        @param firstEntry entry to be put first in sys.path (string)
+        @return path list for use as sys.path (list of strings)
+        """
+        sysPath = [path for path in os.environ.get("PYTHONPATH", "").split(os.pathsep) 
+                   if path not in sys.path] + sys.path[:]
+        if "" in sysPath:
+            sysPath.remove("")
+        sysPath.insert(0, firstEntry)
+        sysPath.insert(0, '')
+        return sysPath
