@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2009 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -11,8 +11,7 @@ import os
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import QDialog, QMessageBox
-from PyQt4.QtNetwork import QNetworkAccessManager, QNetworkRequest, \
-    QNetworkProxy
+from PyQt4.QtNetwork import QNetworkAccessManager, QNetworkRequest
 try:
     from PyQt4.QtNetwork import QSslCertificate, QSslConfiguration, QSslSocket, QSsl
     SSL_AVAILABLE = True
@@ -21,6 +20,8 @@ except ImportError:
 
 from KdeQt import KQMessageBox
 
+from E4Network.E4NetworkProxyFactory import E4NetworkProxyFactory
+
 from UI.AuthenticationDialog import AuthenticationDialog
 
 from Helpviewer.HelpLanguagesDialog import HelpLanguagesDialog
@@ -58,7 +59,9 @@ class NetworkAccessManager(QNetworkAccessManager):
         
         self.__schemeHandlers = {}  # dictionary of scheme handlers
         
-        self.__setAccessManagerProxy()
+        self.__proxyFactory = E4NetworkProxyFactory()
+        self.setProxyFactory(self.__proxyFactory)
+        
         self.__setDiskCache()
         self.languagesChanged()
         
@@ -78,7 +81,7 @@ class NetworkAccessManager(QNetworkAccessManager):
                 self.__sslErrors)
         
         self.connect(self, 
-            SIGNAL('proxyAuthenticationRequired(const QNetworkProxy &, QAuthenticator *)'),
+            SIGNAL('proxyAuthenticationRequired(const QNetworkProxy&, QAuthenticator*)'),
             self.__proxyAuthenticationRequired)
         self.connect(self, 
             SIGNAL('authenticationRequired(QNetworkReply *, QAuthenticator *)'), 
@@ -147,38 +150,6 @@ class NetworkAccessManager(QNetworkAccessManager):
         
         return reply
     
-    def __setAccessManagerProxy(self):
-        """
-        Private method  to set the proxy used by the network access manager.
-        """
-        if Preferences.getUI("UseProxy"):
-            host = Preferences.getUI("ProxyHost")
-            if host.isEmpty():
-                KQMessageBox.critical(None,
-                    self.trUtf8("Web Browser"),
-                    self.trUtf8("""Proxy usage was activated"""
-                                """ but no proxy host configured."""))
-                return
-            else:
-                pProxyType = Preferences.getUI("ProxyType")
-                if pProxyType == 0:
-                    proxyType = QNetworkProxy.HttpProxy
-                elif pProxyType == 1:
-                    proxyType = QNetworkProxy.HttpCachingProxy
-                elif pProxyType == 2:
-                    proxyType = QNetworkProxy.Socks5Proxy
-                self.__proxy = QNetworkProxy(proxyType, host, 
-                    Preferences.getUI("ProxyPort"),
-                    Preferences.getUI("ProxyUser"),
-                    Preferences.getUI("ProxyPassword"))
-                if qVersion() >= '4.5.0':
-                    self.__proxy.setCapabilities(QNetworkProxy.Capabilities(
-                        QNetworkProxy.CachingCapability | \
-                        QNetworkProxy.HostNameLookupCapability))
-        else:
-            self.__proxy = QNetworkProxy(QNetworkProxy.NoProxy)
-        self.setProxy(self.__proxy)
-    
     def __authenticationRequired(self, reply, auth):
         """
         Private slot to handle an authentication request.
@@ -230,8 +201,8 @@ class NetworkAccessManager(QNetworkAccessManager):
             if dlg.shallSave():
                 Preferences.setUI("ProxyUser", username)
                 Preferences.setUI("ProxyPassword", password)
-                self.__proxy.setUser(username)
-                self.__proxy.setPassword(password)
+                proxy.setUser(username)
+                proxy.setPassword(password)
     
     def __sslErrors(self, reply, errors):
         """
@@ -311,7 +282,11 @@ class NetworkAccessManager(QNetworkAccessManager):
         """
         result = QString("<p>")
         
-        result.append(cert.subjectInfo(QSslCertificate.CommonName))
+        result.append(self.trUtf8("Name: %1")\
+            .arg(cert.subjectInfo(QSslCertificate.CommonName)))
+        
+        result.append(self.trUtf8("<br/>Organization: %1")\
+            .arg(cert.subjectInfo(QSslCertificate.Organization)))
         
         result.append(self.trUtf8("<br/>Issuer: %1")\
             .arg(cert.issuerInfo(QSslCertificate.CommonName)))
@@ -328,7 +303,6 @@ class NetworkAccessManager(QNetworkAccessManager):
         """
         Public slot to signal a change of preferences.
         """
-        self.__setAccessManagerProxy()
         self.__setDiskCache()
     
     def languagesChanged(self):
