@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2009 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -189,7 +189,7 @@ class PasswordManager(QObject):
             KQMessageBox.critical(None,
                 self.trUtf8("Saving login data"),
                 self.trUtf8("""<p>Login data could not be saved to <b>%1</b></p>"""
-                            """<p>Reason: %2</p>""").arg(loginFile).arg(unicode(err)))
+                            """<p>Reason: %2</p>""").arg(loginFile).arg(str(err)))
             return
     
     def __load(self):
@@ -206,7 +206,7 @@ class PasswordManager(QObject):
                 KQMessageBox.critical(None,
                     self.trUtf8("Loading login data"),
                     self.trUtf8("""<p>Login data could not be loaded from <b>%1</b></p>"""
-                                """<p>Reason: %2</p>""").arg(loginFile).arg(unicode(err)))
+                                """<p>Reason: %2</p>""").arg(loginFile).arg(str(err)))
                 return
             
             data = []
@@ -458,9 +458,12 @@ class PasswordManager(QObject):
             deadElements = set()
             for element in elements:
                 elementMap = element.toMap()
-                name = elementMap[QString("name")].toString()
-                value = elementMap[QString("value")].toString()
-                type_ = elementMap[QString("type")].toString()
+                try:
+                    name = elementMap[QString("name")].toString()
+                    value = elementMap[QString("value")].toString()
+                    type_ = elementMap[QString("type")].toString()
+                except KeyError:
+                    continue
                 if type_ == "password":
                     formHasPasswords = True
                 t = (name, value)
