@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2003 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2003 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -53,18 +53,20 @@ class SyntaxCheckerDialog(QDialog, Ui_SyntaxCheckerDialog):
         self.resultList.sortItems(self.resultList.sortColumn(), 
                                   self.resultList.header().sortIndicatorOrder())
         
-    def __createResultItem(self, file, line, error, sourcecode):
+    def __createResultItem(self, file, line, index, error, sourcecode):
         """
         Private method to create an entry in the result list.
         
-        @param file filename of file (string or QString)
-        @param line linenumber of faulty source (integer or string)
+        @param file file name of file (string or QString)
+        @param line line number of faulty source (integer or string)
+        @param index index number of fault (integer)
         @param error error text (string or QString)
         @param sourcecode faulty line of code (string)
         """
         itm = QTreeWidgetItem(self.resultList, QStringList() \
             << file << str(line) << error << sourcecode)
         itm.setTextAlignment(1, Qt.AlignRight)
+        itm.setData(0, Qt.UserRole, QVariant(int(index)))
         
     def start(self, fn, codestring = ""):
         """
@@ -97,10 +99,10 @@ class SyntaxCheckerDialog(QDialog, Ui_SyntaxCheckerDialog):
                 if self.cancelled:
                     return
                 
-                nok, fname, line, code, error = Utilities.compile(file, codestring)
+                nok, fname, line, index, code, error = Utilities.compile(file, codestring)
                 if nok:
                     self.noResults = False
-                    self.__createResultItem(fname, line, error, code)
+                    self.__createResultItem(fname, line, index, error, code)
                 progress += 1
                 self.checkProgress.setValue(progress)
                 QApplication.processEvents()
@@ -120,7 +122,7 @@ class SyntaxCheckerDialog(QDialog, Ui_SyntaxCheckerDialog):
         self.buttonBox.button(QDialogButtonBox.Close).setDefault(True)
         
         if self.noResults:
-            self.__createResultItem(self.trUtf8('No syntax errors found.'), "", "", "")
+            self.__createResultItem(self.trUtf8('No syntax errors found.'), "", 0, "", "")
             QApplication.processEvents()
             self.showButton.setEnabled(False)
             self.__clearErrors()
@@ -155,11 +157,12 @@ class SyntaxCheckerDialog(QDialog, Ui_SyntaxCheckerDialog):
         fn = Utilities.normabspath(unicode(itm.text(0)))
         lineno = int(str(itm.text(1)))
         error = unicode(itm.text(2))
+        index = itm.data(0, Qt.UserRole).toInt()[0]
         
         vm = e4App().getObject("ViewManager")
         vm.openSourceFile(fn, lineno)
         editor = vm.getOpenEditor(fn)
-        editor.toggleSyntaxError(lineno, True, error)
+        editor.toggleSyntaxError(lineno, index, True, error, show = True)
         
     @pyqtSignature("")
     def on_showButton_clicked(self):
