@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2004 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2004 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
-Module implementing a dialog to enter the parameters for eric4-api.
+Module implementing a dialog to enter the parameters for eric4_api.
 """
 
 import sys
@@ -26,7 +26,7 @@ from eric4config import getConfig
 
 class EricapiConfigDialog(QDialog, Ui_EricapiConfigDialog):
     """
-    Class implementing a dialog to enter the parameters for eric4-api.
+    Class implementing a dialog to enter the parameters for eric4_api.
     """
     def __init__(self, project, parms = None, parent = None):
         """
@@ -115,7 +115,7 @@ class EricapiConfigDialog(QDialog, Ui_EricapiConfigDialog):
         
         # 1. the program name
         args.append(sys.executable)
-        args.append(Utilities.normabsjoinpath(getConfig('ericDir'), "eric4-api.py"))
+        args.append(Utilities.normabsjoinpath(getConfig('ericDir'), "eric4_api.py"))
         
         # 2. the commandline options
         if self.parameters['outputFile'] != self.defaults['outputFile']:
@@ -175,7 +175,7 @@ class EricapiConfigDialog(QDialog, Ui_EricapiConfigDialog):
         if not filename.isNull():
             # make it relative, if it is in a subdirectory of the project path 
             fn = unicode(Utilities.toNativeSeparators(filename))
-            fn = fn.replace(self.ppath+os.sep, '')
+            fn = self.project.getRelativePath(fn)
             self.outputFileEdit.setText(fn)
 
     def on_outputFileEdit_textChanged(self, filename):
@@ -206,7 +206,7 @@ class EricapiConfigDialog(QDialog, Ui_EricapiConfigDialog):
         if not directory.isNull():
             # make it relative, if it is a subdirectory of the project path 
             dn = unicode(Utilities.toNativeSeparators(directory))
-            dn = dn.replace(self.ppath+os.sep, '')
+            dn = self.project.getRelativePath(dn)
             while dn.endswith(os.sep):
                 dn = dn[:-1]
             self.ignoreDirEdit.setText(dn)
@@ -219,8 +219,10 @@ class EricapiConfigDialog(QDialog, Ui_EricapiConfigDialog):
         The directory in the ignore directories
         line edit is moved to the listbox above and the edit is cleared.
         """
-        self.ignoreDirsList.addItem(os.path.basename(unicode(self.ignoreDirEdit.text())))
-        self.ignoreDirEdit.clear()
+        basename = os.path.basename(unicode(self.ignoreDirEdit.text()))
+        if basename:
+            self.ignoreDirsList.addItem(basename)
+            self.ignoreDirEdit.clear()
 
     @pyqtSignature("")
     def on_deleteButton_clicked(self):
