@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2003 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2003 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -20,6 +20,8 @@ from KdeQt.KQApplication import e4App
 
 from SvnConst import svnStatusMap
 from SvnDialogMixin import SvnDialogMixin
+from SvnDiffDialog import SvnDiffDialog
+
 from Ui_SvnStatusDialog import Ui_SvnStatusDialog
 
 import Preferences
@@ -39,11 +41,16 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         self.setupUi(self)
         SvnDialogMixin.__init__(self)
         
-        self.__changelistColumn = 0
-        self.__statusColumn = 1
-        self.__propStatusColumn = 2
-        self.__lockinfoColumn = 6
-        self.__pathColumn = 11
+        self.__toBeCommittedColumn = 0
+        self.__changelistColumn = 1
+        self.__statusColumn = 2
+        self.__propStatusColumn = 3
+        self.__lockedColumn = 4
+        self.__historyColumn = 5
+        self.__switchedColumn = 6
+        self.__lockinfoColumn = 7
+        self.__upToDateColumn = 8
+        self.__pathColumn = 12
         self.__lastColumn = self.statusList.columnCount()
         
         self.refreshButton = \
@@ -53,6 +60,7 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         self.buttonBox.button(QDialogButtonBox.Close).setEnabled(False)
         self.buttonBox.button(QDialogButtonBox.Cancel).setDefault(True)
         
+        self.diff = None
         self.vcs = vcs
         self.connect(self.vcs, SIGNAL("committed()"), self.__committed)
         
@@ -68,8 +76,12 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         self.menu.addSeparator()
         self.menuactions.append(self.menu.addAction(\
             self.trUtf8("Add to repository"), self.__add))
+        self.menuactions.append(self.menu.addAction(\
+            self.trUtf8("Show differences"), self.__diff))
         self.menuactions.append(self.menu.addAction(\
             self.trUtf8("Revert changes"), self.__revert))
+        self.menuactions.append(self.menu.addAction(\
+            self.trUtf8("Restore missing"), self.__restoreMissing))
         if pysvn.svn_version >= (1, 5, 0) and pysvn.version >= (1, 6, 0):
             self.menu.addSeparator()
             self.menuactions.append(self.menu.addAction(
@@ -98,15 +110,15 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
                      self.__showContextMenu)
         
         self.modifiedIndicators = QStringList()
-        self.modifiedIndicators.append(\
+        self.modifiedIndicators.append(
             self.trUtf8(svnStatusMap[pysvn.wc_status_kind.added]))
-        self.modifiedIndicators.append(\
+        self.modifiedIndicators.append(
             self.trUtf8(svnStatusMap[pysvn.wc_status_kind.deleted]))
-        self.modifiedIndicators.append(\
+        self.modifiedIndicators.append(
             self.trUtf8(svnStatusMap[pysvn.wc_status_kind.modified]))
         
         self.unversionedIndicators = QStringList()
-        self.unversionedIndicators.append(\
+        self.unversionedIndicators.append(
             self.trUtf8(svnStatusMap[pysvn.wc_status_kind.unversioned]))
         
         self.lockedIndicators = QStringList()
@@ -120,6 +132,10 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         self.unlockedIndicators = QStringList()
         self.unlockedIndicators.append(self.trUtf8('not locked'))
         
+        self.missingIndicators = QStringList()
+        self.missingIndicators.append(
+            self.trUtf8(svnStatusMap[pysvn.wc_status_kind.missing]))
+        
         self.lockinfo = {
             ' ' : self.trUtf8('not locked'),
             'L' : self.trUtf8('locked'),
@@ -172,10 +188,12 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         @param author author of the last change (string or QString)
         @param path path of the file or directory (string or QString)
         """
+        statusText = self.trUtf8(svnStatusMap[status])
         itm = QTreeWidgetItem(self.statusList, 
             QStringList() \
+            << "" \
             << changelist \
-            << self.trUtf8(svnStatusMap[status]) \
+            << statusText \
             << self.trUtf8(svnStatusMap[propStatus]) \
             << self.yesno[locked] \
             << self.yesno[history] \
@@ -188,18 +206,29 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
             << path
         )
         
-        itm.setTextAlignment(0, Qt.AlignLeft)
-        itm.setTextAlignment(1, Qt.AlignHCenter)
+        itm.setTextAlignment(1, Qt.AlignLeft)
         itm.setTextAlignment(2, Qt.AlignHCenter)
         itm.setTextAlignment(3, Qt.AlignHCenter)
         itm.setTextAlignment(4, Qt.AlignHCenter)
         itm.setTextAlignment(5, Qt.AlignHCenter)
         itm.setTextAlignment(6, Qt.AlignHCenter)
         itm.setTextAlignment(7, Qt.AlignHCenter)
-        itm.setTextAlignment(8, Qt.AlignRight)
+        itm.setTextAlignment(8, Qt.AlignHCenter)
         itm.setTextAlignment(9, Qt.AlignRight)
-        itm.setTextAlignment(10, Qt.AlignLeft)
+        itm.setTextAlignment(10, Qt.AlignRight)
         itm.setTextAlignment(11, Qt.AlignLeft)
+        itm.setTextAlignment(12, Qt.AlignLeft)
+        
+        if status in [pysvn.wc_status_kind.added,
+                      pysvn.wc_status_kind.deleted,
+                      pysvn.wc_status_kind.modified] or \
+           propStatus in [pysvn.wc_status_kind.added,
+                          pysvn.wc_status_kind.deleted,
+                          pysvn.wc_status_kind.modified]:
+            itm.setCheckState(self.__toBeCommittedColumn, Qt.Checked)
+        
+        if statusText not in self.__statusFilters:
+            self.__statusFilters.append(statusText)
         
     def start(self, fn):
         """
@@ -210,6 +239,18 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         """
         self.errorGroup.hide()
         
+        for act in self.menuactions:
+            act.setEnabled(False)
+        
+        self.addButton.setEnabled(False)
+        self.commitButton.setEnabled(False)
+        self.diffButton.setEnabled(False)
+        self.revertButton.setEnabled(False)
+        self.restoreButton.setEnabled(False)
+
+        self.statusFilterCombo.clear()
+        self.__statusFilters = []
+
         QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
         QApplication.processEvents()
         
@@ -230,6 +271,13 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         recurse = "--non-recursive" not in opts
         update = "--show-updates" in opts
         
+        hideChangelistColumn = True
+        hidePropertyStatusColumn = True
+        hideLockColumns = True
+        hideUpToDateColumn = True
+        hideHistoryColumn = True
+        hideSwitchedColumn = True
+        
         locker = QMutexLocker(self.vcs.vcsExecutionMutex)
         cwd = os.getcwd()
         os.chdir(self.dname)
@@ -245,8 +293,8 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
                     changelists = self.client.get_changelist(name, depth = depth)
                     for entry in changelists:
                         changelistsDict[entry[0]] = entry[1]
-                self.statusList.setColumnHidden(self.__changelistColumn, 
-                                                len(changelistsDict) == 0)
+                hideChangelistColumn = hideChangelistColumn and \
+                    len(changelistsDict) == 0
                 
                 # step 2: determine status of files
                 allFiles = self.client.status(name, recurse = recurse, get_all = verbose, 
@@ -278,7 +326,18 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
                     else:
                         changelist = ""
                     
-                    self.__generateItem(\
+                    hidePropertyStatusColumn = hidePropertyStatusColumn and \
+                        file.prop_status in [
+                            pysvn.wc_status_kind.none, 
+                            pysvn.wc_status_kind.normal
+                        ]
+                    hideLockColumns = hideLockColumns and \
+                        not file.is_locked and lockState == " "
+                    hideUpToDateColumn = hideUpToDateColumn and uptodate
+                    hideHistoryColumn = hideHistoryColumn and not file.is_copied
+                    hideSwitchedColumn = hideSwitchedColumn and not file.is_switched
+                    
+                    self.__generateItem(
                         changelist, 
                         file.text_status, 
                         file.prop_status,
@@ -302,6 +361,15 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
                     break
         except pysvn.ClientError, e:
             self.__showError(e.args[0]+'\n')
+        
+        self.statusList.setColumnHidden(self.__propStatusColumn, hidePropertyStatusColumn)
+        self.statusList.setColumnHidden(self.__lockedColumn, hideLockColumns)
+        self.statusList.setColumnHidden(self.__lockinfoColumn, hideLockColumns)
+        self.statusList.setColumnHidden(self.__upToDateColumn, hideUpToDateColumn)
+        self.statusList.setColumnHidden(self.__historyColumn, hideHistoryColumn)
+        self.statusList.setColumnHidden(self.__switchedColumn, hideSwitchedColumn)
+        self.statusList.setColumnHidden(self.__changelistColumn, hideChangelistColumn)
+        
         locker.unlock()
         self.__finish()
         os.chdir(cwd)
@@ -317,11 +385,16 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         self.buttonBox.button(QDialogButtonBox.Close).setDefault(True)
         
         self.refreshButton.setEnabled(True)
+        self.__updateButtons()
+        self.__updateCommitButton()
+        
+        self.__statusFilters.sort()
+        self.__statusFilters.insert(0, "<%s>" % self.trUtf8("all"))
+        self.statusFilterCombo.addItems(self.__statusFilters)
         
         for act in self.menuactions:
             act.setEnabled(True)
         
-        self.statusList.doItemsLayout()
         self.__resizeColumns()
         self.__resort()
         
@@ -351,9 +424,6 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         
         self.refreshButton.setEnabled(False)
         
-        for act in self.menuactions:
-            act.setEnabled(False)
-        
         self.statusList.clear()
         
         self.shouldCancel = False
@@ -369,6 +439,95 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         self.errors.insertPlainText(msg)
         self.errors.ensureCursorVisible()
     
+    @pyqtSignature("")
+    def on_statusList_itemSelectionChanged(self):
+        """
+        Private slot to act upon changes of selected items.
+        """
+        self.__updateButtons()
+    
+    @pyqtSignature("QTreeWidgetItem*, int")
+    def on_statusList_itemChanged(self, item, column):
+        """
+        Private slot to act upon item changes.
+        
+        @param item reference to the changed item (QTreeWidgetItem)
+        @param column index of column that changed (integer)
+        """
+        if column == self.__toBeCommittedColumn:
+            self.__updateCommitButton()
+
+    def __updateButtons(self):
+        """
+        Private method to update the VCS buttons status.
+        """
+        modified = len(self.__getModifiedItems())
+        unversioned = len(self.__getUnversionedItems())
+        missing = len(self.__getMissingItems())
+
+        self.addButton.setEnabled(unversioned)
+        self.diffButton.setEnabled(modified)
+        self.revertButton.setEnabled(modified)
+        self.restoreButton.setEnabled(missing)
+    
+    def __updateCommitButton(self):
+        """
+        Private method to update the Commit button status.
+        """
+        commitable = len(self.__getCommitableItems())
+        self.commitButton.setEnabled(commitable)
+    
+    @pyqtSignature("")
+    def on_commitButton_clicked(self):
+        """
+        Private slot to handle the press of the Commit button.
+        """
+        self.__commit()
+    
+    @pyqtSignature("")
+    def on_addButton_clicked(self):
+        """
+        Private slot to handle the press of the Add button.
+        """
+        self.__add()
+    
+    @pyqtSignature("")
+    def on_diffButton_clicked(self):
+        """
+        Private slot to handle the press of the Differences button.
+        """
+        self.__diff()
+    
+    @pyqtSignature("")
+    def on_revertButton_clicked(self):
+        """
+        Private slot to handle the press of the Revert button.
+        """
+        self.__revert()
+    
+    @pyqtSignature("")
+    def on_restoreButton_clicked(self):
+        """
+        Private slot to handle the press of the Restore button.
+        """
+        self.__restoreMissing()
+    
+    @pyqtSignature("QString")
+    def on_statusFilterCombo_activated(self, txt):
+        """
+        Private slot to react to the selection of a status filter.
+        
+        @param txt selected status filter (QString)
+        """
+        if txt == "<%s>" % self.trUtf8("all"):
+            for topIndex in range(self.statusList.topLevelItemCount()):
+                topItem = self.statusList.topLevelItem(topIndex)
+                topItem.setHidden(False)
+        else:
+            for topIndex in range(self.statusList.topLevelItemCount()):
+                topItem = self.statusList.topLevelItem(topIndex)
+                topItem.setHidden(topItem.text(self.__statusColumn) != txt)
+    
     ############################################################################
     ## Context menu handling methods
     ############################################################################
@@ -386,11 +545,11 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         Private slot to handle the Commit context menu entry.
         """
         names = [os.path.join(self.dname, unicode(itm.text(self.__pathColumn))) \
-                 for itm in self.__getModifiedItems()]
+                 for itm in self.__getCommitableItems()]
         if not names:
             KQMessageBox.information(self,
                 self.trUtf8("Commit"),
-                self.trUtf8("""There are no uncommitted changes available/selected."""))
+                self.trUtf8("""There are no items selected to be committed."""))
             return
         
         if Preferences.getVCS("AutoSaveFiles"):
@@ -442,6 +601,40 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
         self.on_refreshButton_clicked()
         self.vcs.checkVCSStatus()
         
+    def __restoreMissing(self):
+        """
+        Private slot to handle the Restore Missing context menu entry.
+        """
+        names = [os.path.join(self.dname, unicode(itm.text(self.__pathColumn))) \
+                 for itm in self.__getMissingItems()]
+        if not names:
+            KQMessageBox.information(self,
+                self.trUtf8("Revert"),
+                self.trUtf8("""There are no missing items available/selected."""))
+            return
+        
+        self.vcs.vcsRevert(names)
+        self.on_refreshButton_clicked()
+        self.vcs.checkVCSStatus()
+        
+    def __diff(self):
+        """
+        Private slot to handle the Diff context menu entry.
+        """
+        names = [os.path.join(self.dname, unicode(itm.text(self.__pathColumn))) \
+                 for itm in self.__getModifiedItems()]
+        if not names:
+            KQMessageBox.information(self,
+                self.trUtf8("Differences"),
+                self.trUtf8("""There are no uncommitted changes available/selected."""))
+            return
+        
+        if self.diff is None:
+            self.diff = SvnDiffDialog(self.vcs)
+        self.diff.show()
+        QApplication.processEvents()
+        self.diff.start(names)
+        
     def __lock(self):
         """
         Private slot to handle the Lock context menu entry.
@@ -536,7 +729,20 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
             return
         self.vcs.svnRemoveFromChangelist(names)
         self.on_refreshButton_clicked()
-
+    
+    def __getCommitableItems(self):
+        """
+        Private method to retrieve all entries the user wants to commit.
+        
+        @return list of all items, the user has checked
+        """
+        commitableItems = []
+        for index in range(self.statusList.topLevelItemCount()):
+            itm = self.statusList.topLevelItem(index)
+            if itm.checkState(self.__toBeCommittedColumn) == Qt.Checked:
+                commitableItems.append(itm)
+        return commitableItems
+    
     def __getModifiedItems(self):
         """
         Private method to retrieve all entries, that have a modified status.
@@ -562,6 +768,18 @@ class SvnStatusDialog(QWidget, SvnDialogMixin, Ui_SvnStatusDialog):
                 unversionedItems.append(itm)
         return unversionedItems
         
+    def __getMissingItems(self):
+        """
+        Private method to retrieve all entries, that have a missing status.
+        
+        @return list of all items with a missing status
+        """
+        missingItems = []
+        for itm in self.statusList.selectedItems():
+            if self.missingIndicators.contains(itm.text(self.__statusColumn)):
+                missingItems.append(itm)
+        return missingItems
+        
     def __getLockActionItems(self, indicators):
         """
         Private method to retrieve all entries, that have a locked status.
