@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2003 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2003 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -143,11 +143,11 @@ class SvnDiffDialog(QWidget, Ui_SvnDiffDialog):
             else:
                 dname, fname = self.vcs.splitPath(fn)
                 fnames = [fname]
-            ppath = e4App().getObject('Project').getProjectPath()
-            if dname == ppath:
+            project = e4App().getObject('Project')
+            if dname == project.getProjectPath():
                 path = ""
             else:
-                path = dname.replace(ppath+os.sep, '')
+                path = project.getRelativePath(dname)
             if path:
                 path += "/"
             for fname in fnames:
@@ -311,7 +311,7 @@ class SvnDiffDialog(QWidget, Ui_SvnDiffDialog):
             KQMessageBox.critical(self, self.trUtf8('Save Diff'),
                 self.trUtf8('<p>The patch file <b>%1</b> could not be saved.'
                     '<br>Reason: %2</p>')
-                    .arg(unicode(fname)).arg(unicode(why)))
+                    .arg(unicode(fname)).arg(str(why)))
         
     def on_passwordCheckBox_toggled(self, isOn):
         """
@@ -361,4 +361,4 @@ class SvnDiffDialog(QWidget, Ui_SvnDiffDialog):
             self.intercept = False
             evt.accept()
             return
-        QWidget.keyPressEvent(self, evt)
\ No newline at end of file
+        QWidget.keyPressEvent(self, evt)
