@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2004 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2004 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -336,7 +336,7 @@ class PyRegExpWizardWidget(QWidget, Ui_PyRegExpWizardDialog):
                 KQMessageBox.information(self,
                     self.trUtf8("Save regular expression"),
                     self.trUtf8("""<p>The regular expression could not be saved.</p>"""
-                                """<p>Reason: %1</p>""").arg(unicode(err)))
+                                """<p>Reason: %1</p>""").arg(str(err)))
     
     @pyqtSignature("")
     def on_loadButton_clicked(self):
@@ -359,7 +359,7 @@ class PyRegExpWizardWidget(QWidget, Ui_PyRegExpWizardDialog):
                 KQMessageBox.information(self,
                     self.trUtf8("Save regular expression"),
                     self.trUtf8("""<p>The regular expression could not be saved.</p>"""
-                                """<p>Reason: %1</p>""").arg(unicode(err)))
+                                """<p>Reason: %1</p>""").arg(str(err)))
 
     @pyqtSignature("")
     def on_copyButton_clicked(self):
@@ -406,7 +406,7 @@ class PyRegExpWizardWidget(QWidget, Ui_PyRegExpWizardDialog):
                 KQMessageBox.critical(None,
                     self.trUtf8("Error"),
                     self.trUtf8("""Invalid regular expression: %1""")
-                        .arg(unicode(e)))
+                        .arg(str(e)))
                 return
             except IndexError:
                 KQMessageBox.critical(None,
@@ -533,7 +533,7 @@ class PyRegExpWizardWidget(QWidget, Ui_PyRegExpWizardDialog):
                 KQMessageBox.critical(None,
                     self.trUtf8("Error"),
                     self.trUtf8("""Invalid regular expression: %1""")
-                        .arg(unicode(e)))
+                        .arg(str(e)))
                 return
             except IndexError:
                 KQMessageBox.critical(None,
