@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -58,19 +58,29 @@ class ConfigurationWidget(QWidget):
     
     @signal preferencesChanged emitted after settings have been changed
     """
-    def __init__(self, parent = None, fromEric = True, helpBrowserMode = False):
+    DefaultMode     = 0
+    HelpBrowserMode = 1
+    TrayStarterMode = 2
+    
+    def __init__(self, parent = None, fromEric = True, displayMode = DefaultMode):
         """
         Constructor
         
         @param parent The parent widget of this dialog. (QWidget)
         @keyparam fromEric flag indicating a dialog generation from within the 
             eric4 ide (boolean)
-        @keyparam helpBrowserMode flag indicating to show only help pages
-            for entries related to the help browser (boolean)
+        @keyparam displayMode mode of the configuration dialog 
+            (DefaultMode, HelpBrowserMode, TrayStarterMode)
         """
+        assert displayMode in (
+            ConfigurationWidget.DefaultMode, 
+            ConfigurationWidget.HelpBrowserMode, 
+            ConfigurationWidget.TrayStarterMode
+        )
+        
         QWidget.__init__(self, parent)
         self.fromEric = fromEric
-        self.helpBrowserMode = helpBrowserMode
+        self.displayMode = displayMode
         
         self.__setupUi()
         
@@ -84,7 +94,7 @@ class ConfigurationWidget(QWidget):
                 self.pluginManager = PluginManager(self)
                 e4App().registerObject("PluginManager", self.pluginManager)
         
-        if not helpBrowserMode:
+        if displayMode == ConfigurationWidget.DefaultMode:
             self.configItems = {
                 # key : [display string, pixmap name, dialog module name or 
                 #        page creation function, parent key,
@@ -134,6 +144,9 @@ class ConfigurationWidget(QWidget):
                 "terminalPage" : \
                     [self.trUtf8("Terminal"), "terminal.png",
                     "TerminalPage", None, None],
+                "trayStarterPage" : \
+                    [self.trUtf8("Tray Starter"), "erict.png",
+                    "TrayStarterPage", None, None],
                 "vcsPage" : \
                     [self.trUtf8("Version Control Systems"), "preferences-vcs.png",
                     "VcsPage", None, None],
@@ -206,6 +219,9 @@ class ConfigurationWidget(QWidget):
                     [self.trUtf8("Styles"), 
                     "preferences-highlighting-styles.png",
                     "EditorHighlightingStylesPage", "1editorLexerPage", None],
+                "editorKeywordsPage" : \
+                    [self.trUtf8("Keywords"), "preferences-keywords.png", 
+                     "EditorKeywordsPage", "1editorLexerPage", None], 
                 "editorPropertiesPage" : \
                     [self.trUtf8("Properties"), "preferences-properties.png",
                     "EditorPropertiesPage", "1editorLexerPage", None],
@@ -224,7 +240,7 @@ class ConfigurationWidget(QWidget):
                     [self.trUtf8("Help Viewers"), "preferences-helpviewers.png",
                     "HelpViewersPage", "0helpPage", None],
                 "helpWebBrowserPage" : \
-                    [self.trUtf8("Eric Web Browser"), "ericWeb.png",
+                    [self.trUtf8("eric4 Web Browser"), "ericWeb.png",
                     "HelpWebBrowserPage", "0helpPage", None],
                 
                 "0projectPage" : \
@@ -253,7 +269,7 @@ class ConfigurationWidget(QWidget):
             
             self.configItems.update(
                 e4App().getObject("PluginManager").getPluginConfigData())
-        else:
+        elif displayMode == ConfigurationWidget.HelpBrowserMode:
             self.configItems = {
                 # key : [display string, pixmap name, dialog module name or 
                 #        page creation function, parent key,
@@ -264,9 +280,9 @@ class ConfigurationWidget(QWidget):
                 "networkPage" : \
                     [self.trUtf8("Network"), "preferences-network.png", 
                      "NetworkPage", None, None], 
-                "pythonPage" : \
-                    [self.trUtf8("Python"), "preferences-python.png",
-                    "PythonPage", None, None],
+                "printerPage" : \
+                    [self.trUtf8("Printer"), "preferences-printer.png",
+                    "PrinterPage", None, None],
                 
                 "0helpPage" : \
                     [self.trUtf8("Help"), "preferences-help.png",
@@ -282,9 +298,23 @@ class ConfigurationWidget(QWidget):
                     [self.trUtf8("Help Viewers"), "preferences-helpviewers.png",
                     "HelpViewersPage", "0helpPage", None],
                 "helpWebBrowserPage" : \
-                    [self.trUtf8("Eric Web Browser"), "ericWeb.png",
+                    [self.trUtf8("eric4 Web Browser"), "ericWeb.png",
                     "HelpWebBrowserPage", "0helpPage", None],
             }
+        elif displayMode == ConfigurationWidget.TrayStarterMode:
+            self.configItems = {
+                # key : [display string, pixmap name, dialog module name or 
+                #        page creation function, parent key,
+                #        reference to configuration page (must always be last)]
+                # The dialog module must have the module function create to create
+                # the configuration page. This must have the method save to save 
+                # the settings.
+                "trayStarterPage" : \
+                    [self.trUtf8("Tray Starter"), "erict.png",
+                    "TrayStarterPage", None, None],
+            }
+        else:
+            raise RuntimeError("Illegal mode value: %d" % displayMode)
         
         # generate the list entries
         itemsToExpand = []
@@ -308,7 +338,8 @@ class ConfigurationWidget(QWidget):
         self.connect(self.configList, SIGNAL("itemClicked(QTreeWidgetItem *, int)"),
             self.__showConfigurationPage)
         
-        self.__initLexers()
+        if displayMode != ConfigurationWidget.TrayStarterMode:
+            self.__initLexers()
         
     def __setupUi(self):
         """
@@ -373,7 +404,7 @@ class ConfigurationWidget(QWidget):
             QDialogButtonBox.Apply | QDialogButtonBox.Cancel | \
             QDialogButtonBox.Ok | QDialogButtonBox.Reset)
         self.buttonBox.setObjectName("buttonBox")
-        if not self.fromEric and not self.helpBrowserMode:
+        if not self.fromEric and self.displayMode == ConfigurationWidget.DefaultMode:
             self.buttonBox.button(QDialogButtonBox.Apply).hide()
         self.buttonBox.button(QDialogButtonBox.Apply).setEnabled(False)
         self.buttonBox.button(QDialogButtonBox.Reset).setEnabled(False)
@@ -451,6 +482,10 @@ class ConfigurationWidget(QWidget):
         if page is not None:
             self.configStack.addWidget(page)
             pageData[-1] = page
+            try:
+                page.setMode(self.displayMode)
+            except AttributeError:
+                pass
         return page
         
     def showConfigurationPageByName(self, pageName):
@@ -582,8 +617,12 @@ class ConfigurationDialog(QDialog):
     
     @signal preferencesChanged emitted after settings have been changed
     """
+    DefaultMode     = ConfigurationWidget.DefaultMode
+    HelpBrowserMode = ConfigurationWidget.HelpBrowserMode
+    TrayStarterMode = ConfigurationWidget.TrayStarterMode
+    
     def __init__(self, parent = None, name = None, modal = False, 
-                 fromEric = True, helpBrowserMode = False):
+                 fromEric = True, displayMode = ConfigurationWidget.DefaultMode):
         """
         Constructor
         
@@ -592,8 +631,8 @@ class ConfigurationDialog(QDialog):
         @param modal Flag indicating a modal dialog. (boolean)
         @keyparam fromEric flag indicating a dialog generation from within the 
             eric4 ide (boolean)
-        @keyparam helpBrowserMode flag indicating to show only help pages
-            for entries related to the help browser (boolean)
+        @keyparam displayMode mode of the configuration dialog 
+            (DefaultMode, HelpBrowserMode, TrayStarterMode)
         """
         QDialog.__init__(self, parent)
         if name:
@@ -604,7 +643,7 @@ class ConfigurationDialog(QDialog):
         self.layout.setSpacing(0)
         
         self.cw = ConfigurationWidget(self, fromEric = fromEric, 
-                                      helpBrowserMode = helpBrowserMode)
+                                      displayMode = displayMode)
         size = self.cw.size()
         self.layout.addWidget(self.cw)
         self.resize(size)
