@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2006 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2006 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -141,9 +141,31 @@ class EditorStylesPage(ConfigurationPageBase, Ui_EditorStylesPage):
         
         self.whitespaceCheckBox.setChecked(\
             Preferences.getEditor("ShowWhitespace"))
+        self.whitespaceSizeSpinBox.setValue(
+            Preferences.getEditor("WhitespaceSize"))
+        self.editorColours["WhitespaceForeground"] = \
+            self.initColour("WhitespaceForeground", self.whitespaceForegroundButton, 
+                Preferences.getEditorColour)
+        self.editorColours["WhitespaceBackground"] = \
+            self.initColour("WhitespaceBackground", self.whitespaceBackgroundButton,
+                Preferences.getEditorColour)
+        if not hasattr(QsciScintilla, "setWhitespaceForegroundColor"):
+            self.whitespaceSizeSpinBox.setEnabled(False)
+            self.whitespaceForegroundButton.setEnabled(False)
+            self.whitespaceBackgroundButton.setEnabled(False)
+        
         self.miniMenuCheckBox.setChecked(\
             Preferences.getEditor("MiniContextMenu"))
         
+        self.editAreaOverrideCheckBox.setChecked(
+            Preferences.getEditor("OverrideEditAreaColours"))
+        self.editorColours["EditAreaForeground"] = \
+            self.initColour("EditAreaForeground", self.editAreaForegroundButton,
+                Preferences.getEditorColour)
+        self.editorColours["EditAreaBackground"] = \
+            self.initColour("EditAreaBackground", self.editAreaBackgroundButton,
+                Preferences.getEditorColour)
+        
     def save(self):
         """
         Public slot to save the Editor Styles configuration.
@@ -192,9 +214,15 @@ class EditorStylesPage(ConfigurationPageBase, Ui_EditorStylesPage):
         
         Preferences.setEditor("ShowWhitespace", 
             int(self.whitespaceCheckBox.isChecked()))
+        Preferences.setEditor("WhitespaceSize", 
+            self.whitespaceSizeSpinBox.value())
+        
         Preferences.setEditor("MiniContextMenu",
             int(self.miniMenuCheckBox.isChecked()))
         
+        Preferences.setEditor("OverrideEditAreaColours",
+            int(self.editAreaOverrideCheckBox.isChecked()))
+        
         for key in self.editorColours.keys():
             Preferences.setEditorColour(key, self.editorColours[key])
         
@@ -352,6 +380,42 @@ class EditorStylesPage(ConfigurationPageBase, Ui_EditorStylesPage):
         self.marginsFontSample.setFont(self.marginsFont)
         self.defaultFontSample.setFont(self.defaultFont)
         self.monospacedFontSample.setFont(self.monospacedFont)
+    
+    @pyqtSignature("")
+    def on_whitespaceForegroundButton_clicked(self):
+        """
+        Private slot to set the foreground colour of visible whitespace.
+        """
+        self.editorColours["WhitespaceForeground"] = \
+            self.selectColour(self.whitespaceForegroundButton, 
+                self.editorColours["WhitespaceForeground"])
+    
+    @pyqtSignature("")
+    def on_whitespaceBackgroundButton_clicked(self):
+        """
+        Private slot to set the background colour of visible whitespace.
+        """
+        self.editorColours["WhitespaceBackground"] = \
+            self.selectColour(self.whitespaceBackgroundButton, 
+                self.editorColours["WhitespaceBackground"])
+    
+    @pyqtSignature("")
+    def on_editAreaForegroundButton_clicked(self):
+        """
+        Private slot to set the foreground colour of the edit area.
+        """
+        self.editorColours["EditAreaForeground"] = \
+            self.selectColour(self.editAreaForegroundButton,
+                self.editorColours["EditAreaForeground"])
+        
+    @pyqtSignature("")
+    def on_editAreaBackgroundButton_clicked(self):
+        """
+        Private slot to set the background colour of the edit area.
+        """
+        self.editorColours["EditAreaBackground"] = \
+            self.selectColour(self.editAreaBackgroundButton,
+                self.editorColours["EditAreaBackground"])
 
 def create(dlg):
     """
