@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -89,7 +89,7 @@ class Prefs(object):
         "Style" : "System",
         "StyleSheet" : "",
         "ViewManager" : "tabview",
-        "LayoutType" : "Toolboxes",
+        "LayoutType" : "Sidebars",
         # allowed values are "DockWindows", "FloatingWindows", "Toolboxes" and "Sidebars"
         "LayoutShellEmbedded" : 0,          # 0 = separate
                                             # 1 = embedded in debug browser
@@ -99,6 +99,7 @@ class Prefs(object):
         "BrowsersListFoldersFirst" : 1,
         "BrowsersHideNonPublic" : 0,
         "BrowsersListContentsByOccurrence" : 0, 
+        "BrowsersFileFilters" : "*.py[co];*.so;*.dll",
         "LogViewerAutoRaise" : 1, 
         "SingleApplicationMode" : 0,
         "CaptionShowsFilename" : 1,
@@ -152,7 +153,7 @@ class Prefs(object):
         "ToolbarManagerState" : QtCore.QByteArray(), 
         "ShowSplash" : 1,
         "UseKDEDialogs" : 0, 
-        "SingleCloseButton" : 1, 
+        "SingleCloseButton" : 0, 
         
         "PerformVersionCheck" : 4,      # 0 = off
                                         # 1 = at startup
@@ -160,6 +161,7 @@ class Prefs(object):
                                         # 3 = weekly
                                         # 4 = monthly
         "UseProxy" : 0,
+        "UseSystemProxy" : 1, 
         "ProxyHost" : "",
         "ProxyPort" : 80,
         "ProxyUser" : "",
@@ -208,6 +210,7 @@ class Prefs(object):
         "ConvertTabsOnLoad" : 0,
         "AutomaticEOLConversion" : 1,
         "ShowWhitespace" : 0,
+        "WhitespaceSize" : 1, 
         "ShowEOL" : 0,
         "UseMonospacedFont" : 0,
         "WrapLongLines" : 0,
@@ -215,6 +218,7 @@ class Prefs(object):
         "ClearBreaksOnClose" : 1,
         "StripTrailingWhitespace" : 0, 
         "CommentColumn0" : 1, 
+        "OverrideEditAreaColours": 0,
         
         "EdgeMode" : Qsci.QsciScintilla.EdgeNone,
         "EdgeColumn" : 80,
@@ -266,6 +270,8 @@ class Prefs(object):
         "DefaultEncoding" : "utf-8",
         "DefaultOpenFilter" : "",
         "DefaultSaveFilter" : "",
+        "AdditionalOpenFilters" : QtCore.QStringList(), 
+        "AdditionalSaveFilters" : QtCore.QStringList(), 
         
         # All (most) lexers
         "AllFoldCompact" : 1,
@@ -284,6 +290,8 @@ class Prefs(object):
         "CppIndentOpeningBrace" : 0,
         "CppIndentClosingBrace" : 0,
         "CppDollarsAllowed" : 1, 
+        "CppStylePreprocessor": 0, 
+        "CppHighlightTripleQuotedStrings": 0,
         
         # CSS specifics
         "CssFoldComment" : 1,
@@ -299,6 +307,8 @@ class Prefs(object):
         "HtmlFoldScriptComments" : 0, 
         "HtmlFoldScriptHeredocs" : 0, 
         "HtmlCaseSensitiveTags" : 0,
+        "HtmlDjangoTemplates": 0,
+        "HtmlMakoTemplates": 0, 
         
         # Pascal specifics
         "PascalFoldComment" : 1, 
@@ -309,6 +319,7 @@ class Prefs(object):
         "PerlFoldComment" : 1,
         "PerlFoldPackages" : 1, 
         "PerlFoldPODBlocks" : 1, 
+        "PerlFoldAtElse": 0,
         
         # PostScript specifics
         "PostScriptTokenize" : 0, 
@@ -319,6 +330,9 @@ class Prefs(object):
         "PovFoldComment" : 1,
         "PovFoldDirectives" : 0,
         
+        # Properties specifics
+        "PropertiesInitialSpaces": 1, 
+        
         # Python specifics
         "PythonBadIndentation" : 1,
         "PythonFoldComment" : 1,
@@ -327,10 +341,29 @@ class Prefs(object):
         "PythonAllowV2Unicode" : 1, 
         "PythonAllowV3Binary" : 1, 
         "PythonAllowV3Bytes" : 1, 
+        "PythonFoldQuotes": 0,
+        "PythonStringsOverNewLineAllowed": 0,
+        "PythonHighlightSubidentifier": 1,
+        
+        # Ruby specifics
+        "RubyFoldComment": 0,
         
         # SQL specifics
         "SqlFoldComment" : 1,
         "SqlBackslashEscapes" : 0,
+        "SqlDottedWords": 0,
+        "SqlFoldAtElse": 0,
+        "SqlFoldOnlyBegin": 0,
+        "SqlHashComments": 0,
+        "SqlQuotedIdentifiers": 0,
+        
+        # TCL specifics
+        "TclFoldComment": 0,
+        
+        # TeX specifics
+        "TexFoldComment": 0,
+        "TexProcessComments": 0,
+        "TexProcessIf": 1,
         
         # VHDL specifics
         "VHDLFoldComment" : 1,
@@ -368,6 +401,12 @@ class Prefs(object):
         "MarginsForeground"    : QtGui.QColor(QtCore.Qt.black),
         "FoldmarginBackground" : QtGui.QColor(230, 230, 230),
         "SpellingMarkers"      : QtGui.QColor(QtCore.Qt.red),
+        "WhitespaceForeground" : QtGui.QColor(QtCore.Qt.darkGray),
+        "WhitespaceBackground" : QtGui.QColor(QtCore.Qt.white),
+        "EditAreaForeground"   : QtGui.QApplication.palette().color(
+            QtGui.QPalette.Active, QtGui.QPalette.Base),
+        "EditAreaBackground"   : QtGui.QApplication.palette().color(
+            QtGui.QPalette.Active, QtGui.QPalette.Text),
     }
     
     editorOtherFontsDefaults = {
@@ -556,6 +595,7 @@ class Prefs(object):
         "SavePasswords" : 0, 
         "AdBlockEnabled" : 0, 
         "AdBlockSubscriptions" : QtCore.QStringList(), 
+        "SearchLanguage": QtCore.QLocale().language(),
     }
     
     websettings = QWebSettings.globalSettings()
@@ -669,6 +709,7 @@ class Prefs(object):
         "TasksBugfixColour"    : QtGui.QColor(QtCore.Qt.red),
         "TasksBgColour"        : QtGui.QColor(QtCore.Qt.white),
         "TasksProjectBgColour" : QtGui.QColor(QtCore.Qt.lightGray),
+        "ClearOnFileClose": 1,
     }
     
     # defaults for templates related stuff
@@ -695,6 +736,12 @@ class Prefs(object):
         "IconEditorState" : QtCore.QByteArray(), 
     }
     
+    # defaults for tray starter
+    trayStarterDefaults = {
+        "TrayStarterIcon" : "erict.png",
+        # valid values are: erict.png, erict-hc.png, erict-bw.png, erict-bwi.png
+    }
+    
     # defaults for geometry
     geometryDefaults = {
         "HelpViewerGeometry" : QtCore.QByteArray(),
@@ -838,14 +885,21 @@ def exportPreferences(prefClass = Prefs):
     """
     from KdeQt import KQFileDialog
 
+    selectedFilter = QtCore.QString('')
     filename = KQFileDialog.getSaveFileName(\
         None,
         QtCore.QCoreApplication.translate("Preferences", "Export Preferences"),
         QtCore.QString(),
-        QtCore.QString(),
-        None,
+        QtCore.QCoreApplication.translate("Preferences", 
+            "Properties File (*.ini);;All Files (*)"),
+        selectedFilter,
         QtGui.QFileDialog.Options(QtGui.QFileDialog.DontConfirmOverwrite))
     if not filename.isEmpty():
+        ext = QtCore.QFileInfo(filename).suffix()
+        if ext.isEmpty():
+            ex = selectedFilter.section('(*',1,1).section(')',0,0)
+            if not ex.isEmpty():
+                filename.append(ex)
         settingsFile = unicode(prefClass.settings.fileName())
         prefClass.settings = None
         shutil.copy(settingsFile, unicode(filename))
@@ -864,7 +918,8 @@ def importPreferences(prefClass = Prefs):
         None,
         QtCore.QCoreApplication.translate("Preferences", "Import Preferences"),
         QtCore.QString(),
-        QtCore.QString(),
+        QtCore.QCoreApplication.translate("Preferences", 
+            "Properties File (*.ini);;All Files (*)"),
         None)
     if not filename.isEmpty():
         settingsFile = unicode(prefClass.settings.fileName())
@@ -1090,7 +1145,7 @@ def getUI(key, prefClass = Prefs):
                 "CaptionShowsFilename", "CaptionFilenameLength",
                 "RecentNumber", "UseKDEDialogs", "ShowSplash", 
                 "PerformVersionCheck", "SingleCloseButton", 
-                "UseProxy", "ProxyPort", "ProxyType", 
+                "UseProxy", "ProxyPort", "ProxyType", "UseSystemProxy", 
                 "TopLeftByLeft", "BottomLeftByLeft", 
                 "TopRightByRight", "BottomRightByRight", 
                 "OpenOnStartup", "RequestDownloadFilename", 
@@ -1100,7 +1155,8 @@ def getUI(key, prefClass = Prefs):
             QtCore.QVariant(prefClass.uiDefaults[key])).toInt()[0]
     if key in ["Style", "StyleSheet", 
                 "ProxyHost", "ProxyUser", 
-                "PluginRepositoryUrl", "DownloadPath"]:
+                "PluginRepositoryUrl", "DownloadPath", 
+                "BrowsersFileFilters"]:
         return prefClass.settings.value("UI/" + key,
             QtCore.QVariant(prefClass.uiDefaults[key])).toString()
     if key == "ProxyPassword":
@@ -1207,6 +1263,9 @@ def getEditor(key, prefClass = Prefs):
                "SpellCheckingPersonalExcludeList"]:
         return prefClass.settings.value("Editor/" + key,
             QtCore.QVariant(prefClass.editorDefaults[key])).toString()
+    elif key in ["AdditionalOpenFilters", "AdditionalSaveFilters"]:
+        return prefClass.settings.value("Editor/" + key, 
+            QtCore.QVariant(prefClass.editorDefaults[key])).toStringList()
     else:
         return prefClass.settings.value("Editor/" + key,
             QtCore.QVariant(prefClass.editorDefaults[key])).toInt()[0]
@@ -1297,6 +1356,30 @@ def setEditorAPI(key, apilist, prefClass = Prefs):
     """
     prefClass.settings.setValue("Editor/APIs/" + key, QtCore.QVariant(apilist))
     
+def getEditorKeywords(key, prefClass = Prefs):
+    """
+    Module function to retrieve the various lists of language keywords.
+    
+    @param key the key of the value to get
+    @param prefClass preferences class used as the storage area
+    @return the requested list of language keywords (QStringList)
+    """
+    keywords = prefClass.settings.value("Editor/Keywords/" + key)
+    if keywords.isValid():
+        return keywords.toStringList()
+    else:
+        return QtCore.QStringList()
+    
+def setEditorKeywords(key, keywordsLists, prefClass = Prefs):
+    """
+    Module function to store the various lists of language keywords.
+    
+    @param key the key of the api to be set
+    @param keywordsLists the list of language keywords (QStringList)
+    @param prefClass preferences class used as the storage area
+    """
+    prefClass.settings.setValue("Editor/Keywords/" + key, QtCore.QVariant(keywordsLists))
+    
 def getEditorLexerAssocs(prefClass = Prefs):
     """
     Module function to retrieve all lexer associations.
@@ -1643,12 +1726,13 @@ def getQt4DocDir(prefClass = Prefs):
     @param prefClass preferences class used as the storage area
     @return the requested Qt4DocDir setting (string)
     """
-    s = prefClass.settings.value("Help/Qt4DocDir",
-        QtCore.QVariant(prefClass.helpDefaults["Qt4DocDir"])).toString()
-    if s.isEmpty():
-        return os.getenv("QT4DOCDIR", "")
-    else:
-        return unicode(s)
+    s = unicode(prefClass.settings.value("Help/Qt4DocDir",
+        QtCore.QVariant(prefClass.helpDefaults["Qt4DocDir"])).toString())
+    if s == "":
+        s = os.getenv("QT4DOCDIR", "")
+    if s == "":
+        s = unicode(QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.DocumentationPath))
+    return s
     
 def getHelp(key, prefClass = Prefs):
     """
@@ -1760,6 +1844,8 @@ def getQt4TranslationsDir(prefClass = Prefs):
         QtCore.QVariant(prefClass.qtDefaults["Qt4TranslationsDir"])).toString())
     if s == "":
         s = os.getenv("QT4TRANSLATIONSDIR", "")
+    if s == "":
+        s = unicode(QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))
     if s == "" and isWindowsPlatform():
         from PyQt4 import pyqtconfig
         transPath = os.path.join(pyqtconfig._pkg_config["pyqt_mod_dir"], "translations")
@@ -1781,8 +1867,11 @@ def getQt(key, prefClass = Prefs):
         return prefClass.settings.value("Qt/" + key, 
             QtCore.QVariant(prefClass.qtDefaults[key])).toString()
     elif key in ["Qt4Dir"]:
-        return unicode(prefClass.settings.value("Qt/" + key, 
+        p = unicode(prefClass.settings.value("Qt/" + key, 
             QtCore.QVariant(prefClass.qtDefaults[key])).toString())
+        if p == "":
+            p = unicode(QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.BinariesPath))
+        return p
     else: 
         return prefClass.settings.value("Qt/" + key, 
             QtCore.QVariant(prefClass.qtDefaults[key])).toInt()[0]
@@ -1889,6 +1978,9 @@ def getTasks(key, prefClass = Prefs):
             return QtGui.QColor(col.toString())
         else:
             return prefClass.tasksDefaults[key]
+    elif key in ["ClearOnFileClose"]:
+        return prefClass.settings.value("Tasks/" + key,
+            QtCore.QVariant(prefClass.tasksDefaults[key])).toInt()[0]
     else:
         return prefClass.settings.value("Tasks/" + key,
             QtCore.QVariant(prefClass.tasksDefaults[key])).toString()
@@ -2011,6 +2103,27 @@ def setIconEditor(key, value, prefClass = Prefs):
     @param prefClass preferences class used as the storage area
     """
     prefClass.settings.setValue("IconEditor/" + key, QtCore.QVariant(value))
+
+def getTrayStarter(key, prefClass = Prefs):
+    """
+    Module function to retrieve the tray starter related settings.
+    
+    @param key the key of the value to get
+    @param prefClass preferences class used as the storage area
+    @return the requested user setting
+    """
+    return prefClass.settings.value("TrayStarter/" + key,
+            QtCore.QVariant(prefClass.trayStarterDefaults[key])).toString()
+    
+def setTrayStarter(key, value, prefClass = Prefs):
+    """
+    Module function to store the tray starter related settings.
+    
+    @param key the key of the setting to be set
+    @param value the value to be set
+    @param prefClass preferences class used as the storage area
+    """
+    prefClass.settings.setValue("TrayStarter/" + key, QtCore.QVariant(value))
     
 def getGeometry(key, prefClass = Prefs):
     """
