@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2006 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2006 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -122,7 +122,11 @@ class ProjectBrowserSimpleDirectoryItem(BrowserItem, ProjectBrowserItemMixin):
             self._dirName = os.path.dirname(self._dirName)
         
         self.type_ = ProjectBrowserItemSimpleDirectory
-        self.icon = UI.PixmapCache.getIcon("dirClosed.png")
+        if os.path.lexists(self._dirName) and os.path.islink(self._dirName):
+            self.symlink = True
+            self.icon = UI.PixmapCache.getSymlinkIcon("dirClosed.png")
+        else:
+            self.icon = UI.PixmapCache.getIcon("dirClosed.png")
     
     def setName(self, dinfo, full = True):
         """
@@ -218,6 +222,11 @@ class ProjectBrowserModel(BrowserModel):
         self.progDir = None
         self.project = parent
         
+        self.watchedItems = {}
+        self.watcher = QFileSystemWatcher(self)
+        self.connect(self.watcher, SIGNAL("directoryChanged(const QString&)"), 
+                     self.directoryChanged)
+        
         self.inRefresh = False
         
         self.projectBrowserTypes = {
@@ -304,6 +313,8 @@ class ProjectBrowserModel(BrowserModel):
         @param parentItem reference to the directory item to be populated
         @param repopulate flag indicating a repopulation (boolean)
         """
+        self._addWatchedItem(parentItem)
+        
         qdir = QDir(parentItem.dirName())
         
         entryInfoList = \
@@ -348,6 +359,11 @@ class ProjectBrowserModel(BrowserModel):
         """
         self.__vcsStatus = {}
         
+        self.watchedItems = {}
+        watchedDirs = self.watcher.directories()
+        if len(watchedDirs) != 0:
+            self.watcher.removePaths(watchedDirs)
+        
         self.rootItem.removeChildren()
         self.reset()
         
@@ -368,22 +384,7 @@ class ProjectBrowserModel(BrowserModel):
                     states[os.path.normcase(os.path.join(self.project.ppath, fn))] = 0
             
             self.project.vcs.clearStatusCache()
-            for dir in self.project.subdirs:
-                states = self.project.vcs.vcsAllRegisteredStates(states, 
-                    os.path.join(self.project.ppath, dir))
-            
-            for dir in self.project.otherssubdirs:
-                if not os.path.isabs(dir):
-                    dir = os.path.join(self.project.ppath, dir)
-                states = self.project.vcs.vcsAllRegisteredStates(states, dir)
-            
-            if self.project.pdata["TRANSLATIONPATTERN"]:
-                dir = os.path.join(self.project.ppath, 
-                                   self.project.pdata["TRANSLATIONPATTERN"][0])\
-                      .split("%language%")[0]
-                if not os.path.isdir(dir):
-                    dir = os.path.dirname(dir)
-                states = self.project.vcs.vcsAllRegisteredStates(states, dir)
+            states = self.project.vcs.vcsAllRegisteredStates(states, self.project.ppath)
         
         self.inRefresh = True
         for key in keys:
@@ -551,7 +552,7 @@ class ProjectBrowserModel(BrowserModel):
         """
         name = QString(name)
         if QDir.isAbsolutePath(name):
-            name = name.replace(self.project.ppath + os.sep, "")
+            name = self.project.getRelativePath(name)
         pathlist = name.split(QRegExp(r'/|\\'))
         if len(pathlist) > 0:
             olditem = self.rootItem
@@ -578,6 +579,79 @@ class ProjectBrowserModel(BrowserModel):
             index =  self.createIndex(itm.row(), 0, itm)
         return index
     
+    def directoryChanged(self, path):
+        """
+        Public slot to handle the directoryChanged signal of the watcher.
+        
+        @param path path of the directory (string)
+        """
+        path = unicode(path)
+        if path not in self.watchedItems:
+            # just ignore the situation we don't have a reference to the item
+            return
+        
+        for itm in self.watchedItems[path]:
+            oldCnt = itm.childCount()
+            
+            qdir = QDir(itm.dirName())
+            
+            entryInfoList = qdir.entryInfoList(
+                QDir.Filters(QDir.AllEntries | QDir.NoDotAndDotDot))
+            
+            # step 1: check for new entries
+            children = itm.children()
+            for f in entryInfoList:
+                fpath = unicode(f.absoluteFilePath())
+                childFound = False
+                for child in children:
+                    if child.name() == fpath:
+                        childFound = True
+                        children.remove(child)
+                        break
+                if childFound:
+                    continue
+                
+                cnt = itm.childCount()
+                self.beginInsertRows(self.createIndex(itm.row(), 0, itm),
+                    cnt, cnt)
+                if f.isDir():
+                    node = ProjectBrowserDirectoryItem(itm,
+                        unicode(Utilities.toNativeSeparators(f.absoluteFilePath())), 
+                        itm.getProjectTypes()[0], 
+                        False)
+                else:
+                    node = ProjectBrowserFileItem(itm,
+                        unicode(Utilities.toNativeSeparators(f.absoluteFilePath())), 
+                        itm.getProjectTypes()[0])
+                self._addItem(node, itm)
+                if self.project.vcs is not None:
+                    self.project.vcs.clearStatusCache()
+                    state = self.project.vcs.vcsRegisteredState(node.name())
+                    if state == self.project.vcs.canBeCommitted:
+                        node.addVcsStatus(self.project.vcs.vcsName())
+                    else:
+                        node.addVcsStatus(self.trUtf8("local"))
+                self.endInsertRows()
+            
+            # step 2: check for removed entries
+            if len(entryInfoList) != itm.childCount():
+                for row in range(oldCnt - 1, -1, -1):
+                    child = itm.child(row)
+                    entryFound = False
+                    for f in entryInfoList:
+                        if unicode(f.absoluteFilePath()) == child.name():
+                            entryFound = True
+                            entryInfoList.remove(f)
+                            break
+                    if entryFound:
+                        continue
+                    
+                    self._removeWatchedItem(child)
+                    self.beginRemoveRows(self.createIndex(itm.row(), 0, itm),
+                        row, row)
+                    itm.removeChild(child)
+                    self.endRemoveRows()
+    
     def __addVCSStatus(self, item, name):
         """
         Private method used to set the vcs status of a node.
@@ -600,7 +674,7 @@ class ProjectBrowserModel(BrowserModel):
         
         @param item item to work on
         @param name filename belonging to this item (string)
-        @param recursive flag indicating a recursive update (boolean)
+        @keyparam recursive flag indicating a recursive update (boolean)
         """
         if self.project.vcs is not None:
             self.project.vcs.clearStatusCache()
