@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -436,7 +436,7 @@ class ProjectResourcesBrowser(ProjectBaseBrowser):
                 KQMessageBox.critical(self,
                     self.trUtf8("New Resource"),
                     self.trUtf8("<p>The new resource file <b>%1</b> could not be created.<br>"
-                        "Problem: %2</p>").arg(fname).arg(unicode(e)))
+                        "Problem: %2</p>").arg(fname).arg(str(e)))
                 return
             
             self.project.appendFile(fname)
@@ -453,7 +453,7 @@ class ProjectResourcesBrowser(ProjectBaseBrowser):
         for itm in itmList:
             fn2 = unicode(itm.fileName())
             fullNames.append(fn2)
-            fn = fn2.replace(self.project.ppath+os.sep, '')
+            fn = self.project.getRelativePath(fn2)
             files.append(fn)
         
         dlg = DeleteFilesConfirmationDialog(self.parent(),
@@ -528,7 +528,7 @@ class ProjectResourcesBrowser(ProjectBaseBrowser):
                     KQMessageBox.information(None,
                         self.trUtf8("Resource Compilation"),
                         self.trUtf8("<p>The compilation of the resource file failed.</p>"
-                            "<p>Reason: %1</p>").arg(unicode(msg)))
+                            "<p>Reason: %1</p>").arg(str(msg)))
         else:
             if not self.noDialog:
                 KQMessageBox.information(None,
@@ -623,7 +623,7 @@ class ProjectResourcesBrowser(ProjectBaseBrowser):
         """
         itm = self.model().item(self.currentIndex())
         fn2 = unicode(itm.fileName())
-        fn = fn2.replace(self.project.ppath+os.sep, '')
+        fn = self.project.getRelativePath(fn2)
         if self.hooks["compileResource"] is not None:
             self.hooks["compileResource"](fn)
         else:
@@ -664,7 +664,7 @@ class ProjectResourcesBrowser(ProjectBaseBrowser):
         Private method to compile selected resources to source files.
         """
         items = self.getSelectedItems()
-        files = [unicode(itm.fileName()).replace(self.project.ppath+os.sep, '') \
+        files = [self.project.getRelativePath(unicode(itm.fileName())) \
                  for itm in items]
         
         if self.hooks["compileSelectedResources"] is not None:
