@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -538,11 +538,13 @@ class ProjectSourcesBrowser(ProjectBaseBrowser):
         for itm in itmList:
             if isinstance(itm, BrowserFileItem):
                 if itm.isPythonFile():
-                    self.emit(SIGNAL('sourceFile'), itm.fileName(), 1, "Python")
+                    self.emit(SIGNAL('sourceFile'), itm.fileName(), -1, "Python")
+                elif itm.isPython3File():
+                    self.emit(SIGNAL('sourceFile'), itm.fileName(), -1, "Python3")
                 elif itm.isRubyFile():
-                    self.emit(SIGNAL('sourceFile'), itm.fileName(), 1, "Ruby")
+                    self.emit(SIGNAL('sourceFile'), itm.fileName(), -1, "Ruby")
                 elif itm.isDFile():
-                    self.emit(SIGNAL('sourceFile'), itm.fileName(), 1, "D")
+                    self.emit(SIGNAL('sourceFile'), itm.fileName(), -1, "D")
                 else:
                     self.emit(SIGNAL('sourceFile'), itm.fileName())
             elif isinstance(itm, BrowserClassItem):
@@ -570,7 +572,7 @@ class ProjectSourcesBrowser(ProjectBaseBrowser):
         else:
             dn = ""
         
-        dn = dn.replace(self.project.ppath, "")
+        dn = self.project.getRelativePath(dn)
         if dn.startswith(os.sep):
             dn = dn[1:]
         dlg = NewPythonPackageDialog(dn, self)
@@ -590,7 +592,7 @@ class ProjectSourcesBrowser(ProjectBaseBrowser):
                             self.trUtf8("""<p>The package directory <b>%1</b> could"""
                                         """ not be created. Aborting...</p>"""
                                         """<p>Reason: %2</p>""")\
-                                .arg(packagePath).arg(unicode(err)),
+                                .arg(packagePath).arg(str(err)),
                             QMessageBox.StandardButtons(\
                                 QMessageBox.Ok))
                         return
@@ -605,7 +607,7 @@ class ProjectSourcesBrowser(ProjectBaseBrowser):
                             self.trUtf8("""<p>The package file <b>%1</b> could"""
                                         """ not be created. Aborting...</p>"""
                                         """<p>Reason: %2</p>""")\
-                                .arg(packageFile).arg(unicode(err)),
+                                .arg(packageFile).arg(str(err)),
                             QMessageBox.StandardButtons(\
                                 QMessageBox.Ok))
                         return
@@ -656,7 +658,7 @@ class ProjectSourcesBrowser(ProjectBaseBrowser):
         for itm in itmList:
             fn2 = unicode(itm.fileName())
             fullNames.append(fn2)
-            fn = fn2.replace(self.project.ppath+os.sep, '')
+            fn = self.project.getRelativePath(fn2)
             files.append(fn)
         
         dlg = DeleteFilesConfirmationDialog(self.parent(),
