@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -240,7 +240,7 @@ class PropertiesDialog(QDialog, Ui_PropertiesDialog):
         self.project.pdata["VERSION"] = [unicode(self.versionEdit.text())]
         fn = self.mainscriptEdit.text()
         if not fn.isEmpty():
-            fn = unicode(fn).replace(self.project.ppath+os.sep, "")
+            fn = self.project.getRelativePath(unicode(fn))
             self.project.pdata["MAINSCRIPT"] = [fn]
             self.project.translationsRoot = os.path.splitext(fn)[0]
         else:
