@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2005 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2005 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -55,3 +55,12 @@ class LexerCSS(QsciLexerCSS, Lexer):
         """
         return style in [QsciLexerCSS.DoubleQuotedString, 
                          QsciLexerCSS.SingleQuotedString]
+    
+    def defaultKeywords(self, kwSet):
+        """
+        Public method to get the default keywords.
+        
+        @param kwSet number of the keyword set (integer)
+        @return string giving the keywords (string) or None
+        """
+        return QsciLexerCSS.keywords(self, kwSet)
