@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2008 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -74,3 +74,13 @@ class LexerContainer(QsciLexer, Lexer):
         """
         self.editor.startStyling(start, 0x1f)
         self.editor.setStyling(end - start + 1, 0)
+    
+    def keywords(self, kwSet):
+        """
+        Public method to get the list of keywords.
+        
+        @param kwSet number of keyword set (integer, one based)
+        @return list of supported keywords (string) or None
+        """
+        
+        return Lexer.keywords(self, kwSet)
