@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2008 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -59,3 +59,12 @@ class LexerFortran77(QsciLexerFortran77, Lexer):
         return style in [QsciLexerFortran77.DoubleQuotedString, 
                          QsciLexerFortran77.SingleQuotedString, 
                          QsciLexerFortran77.UnclosedString]
+    
+    def defaultKeywords(self, kwSet):
+        """
+        Public method to get the default keywords.
+        
+        @param kwSet number of the keyword set (integer)
+        @return string giving the keywords (string) or None
+        """
+        return QsciLexerFortran77.keywords(self, kwSet)
