@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2008 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -262,6 +262,9 @@ class Terminal(QsciScintillaCompat):
         self.setLexer(self.lexer_)
         self.lexer_.readSettings(Preferences.Prefs.settings, "Scintilla")
         
+        self.lexer_.setDefaultColor(self.lexer_.color(0))
+        self.lexer_.setDefaultPaper(self.lexer_.paper(0))
+        
     def __setMargin0(self):
         """
         Private method to configure margin 0.
@@ -290,6 +293,16 @@ class Terminal(QsciScintillaCompat):
         self.setTabWidth(Preferences.getEditor("TabWidth"))
         if Preferences.getEditor("ShowWhitespace"):
             self.setWhitespaceVisibility(QsciScintilla.WsVisible)
+            try:
+                self.setWhitespaceForegroundColor(
+                    Preferences.getEditorColour("WhitespaceForeground"))
+                self.setWhitespaceBackgroundColor(
+                    Preferences.getEditorColour("WhitespaceBackground"))
+                self.setWhitespaceSize(
+                    Preferences.getEditor("WhitespaceSize"))
+            except AttributeError:
+                # QScintilla before 2.5 doesn't support this
+                pass
         else:
             self.setWhitespaceVisibility(QsciScintilla.WsInvisible)
         self.setEolVisibility(Preferences.getEditor("ShowEOL"))
@@ -333,6 +346,10 @@ class Terminal(QsciScintillaCompat):
         
         self.setCursorFlashTime(QApplication.cursorFlashTime())
         
+        if Preferences.getEditor("OverrideEditAreaColours"):
+            self.setColor(Preferences.getEditorColour("EditAreaForeground"))
+            self.setPaper(Preferences.getEditorColour("EditAreaBackground"))
+        
     def __setMonospaced(self, on):
         """
         Private method to set/reset a monospaced font.
