@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2005 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2005 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -36,7 +36,7 @@ class Task(QTreeWidgetItem):
     """
     def __init__(self, description, priority = 1, filename = "", lineno = 0, 
                  completed = False, _time = 0, isProjectTask = False, 
-                 isBugfixTask = False, ppath = "", longtext = ""):
+                 isBugfixTask = False, project = None, longtext = ""):
         """
         Constructor
         
@@ -50,7 +50,7 @@ class Task(QTreeWidgetItem):
         @param isProjectTask flag indicating a task related to the current project
             (boolean)
         @param isBugfixTask flag indicating a bugfix task (boolean)
-        @param ppath the project path (string or QString)
+        @param project reference to the project object (Project)
         @param longtext explanatory text of the task (string or QString)
         """
         self.description = unicode(description)
@@ -65,10 +65,10 @@ class Task(QTreeWidgetItem):
         self.created = _time and _time or time.time()
         self._isProjectTask = isProjectTask
         self.isBugfixTask = isBugfixTask
-        self.ppath = unicode(ppath)
+        self.project = project
         
         if isProjectTask:
-            self.filename = self.filename.replace(self.ppath + os.sep, "")
+            self.filename = self.project.getRelativePath(self.filename)
             
         QTreeWidgetItem.__init__(self,  
             QStringList() << "" << "" << self.description << self.filename \
@@ -169,7 +169,7 @@ class Task(QTreeWidgetItem):
         @return filename (string)
         """
         if self._isProjectTask and self.filename:
-            return os.path.join(self.ppath, self.filename)
+            return os.path.join(self.project.getProjectPath(), self.filename)
         else:
             return self.filename
     
@@ -536,8 +536,7 @@ class TaskViewer(QTreeWidget):
         """
         task = Task(description, priority, filename, lineno, completed, 
                    _time, isProjectTask, isBugfixTask, 
-                   self.project and self.project.ppath or "", 
-                   longtext)
+                   self.project, longtext)
         self.tasks.append(task)
         if self.taskFilter.showTask(task):
             self.addTopLevelItem(task)
@@ -598,13 +597,21 @@ class TaskViewer(QTreeWidget):
                 self.tasks.remove(task)
                 del task
         
-    def clearFileTasks(self, filename):
+    def clearFileTasks(self, filename, conditionally=False):
         """
         Public slot to clear all tasks related to a file.
         
         @param filename name of the file (string or QString)
+        @param conditionally flag indicating to clear the tasks of the file
+            checking some conditions (boolean)
         """
         filename = unicode(filename)
+        if conditionally:
+            if self.project and self.project.isProjectSource(filename):
+                # project related tasks will not be cleared
+                return
+            if not Preferences.getTasks("ClearOnFileClose"):
+                return
         for task in self.tasks[:]:
             if task.getFilename() == filename:
                 if self.copyTask == task:
