@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2006 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2006 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -9,6 +9,7 @@ Module implementing the browser model.
 
 import sys
 import os
+import fnmatch
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -46,6 +47,10 @@ class BrowserModel(QAbstractItemModel):
         self.rootItem = BrowserItem(None, rootData)
         
         self.progDir = None
+        self.watchedItems = {}
+        self.watcher = QFileSystemWatcher(self)
+        self.connect(self.watcher, SIGNAL("directoryChanged(const QString&)"), 
+                     self.directoryChanged)
         
         self.__populateModel()
     
@@ -86,6 +91,12 @@ class BrowserModel(QAbstractItemModel):
         elif role == Qt.DecorationRole:
             if index.column() == 0:
                 return QVariant(index.internalPointer().getIcon())
+        elif role == Qt.FontRole:
+            item = index.internalPointer()
+            if item.isSymlink():
+                font = QFont(QApplication.font("QTreeView"))
+                font.setItalic(True)
+                return QVariant(font)
         
         return QVariant()
     
@@ -229,6 +240,104 @@ class BrowserModel(QAbstractItemModel):
         
         return index.internalPointer()
     
+    def _addWatchedItem(self, itm):
+        """
+        Protected method to watch an item.
+        
+        @param itm item to be watched (BrowserDirectoryItem)
+        """
+        if isinstance(itm, BrowserDirectoryItem):
+            dirName = itm.dirName()
+            if dirName != "" and \
+               not dirName.startswith("//") and \
+               not dirName.startswith("\\\\"):
+                if dirName not in self.watcher.directories():
+                    self.watcher.addPath(dirName)
+                if dirName in self.watchedItems:
+                    if itm not in self.watchedItems[dirName]:
+                        self.watchedItems[dirName].append(itm)
+                else:
+                    self.watchedItems[dirName] = [itm]
+    
+    def _removeWatchedItem(self, itm):
+        """
+        Protected method to remove a watched item.
+        
+        @param itm item to be removed (BrowserDirectoryItem)
+        """
+        if isinstance(itm, BrowserDirectoryItem):
+            dirName = itm.dirName()
+            if dirName in self.watchedItems:
+                if itm in self.watchedItems[dirName]:
+                    self.watchedItems[dirName].remove(itm)
+                if len(self.watchedItems[dirName]) == 0:
+                    del self.watchedItems[dirName]
+                    self.watcher.removePath(dirName)
+    
+    def directoryChanged(self, path):
+        """
+        Public slot to handle the directoryChanged signal of the watcher.
+        
+        @param path path of the directory (string)
+        """
+        path = unicode(path)
+        if path not in self.watchedItems:
+            # just ignore the situation we don't have a reference to the item
+            return
+        
+        for itm in self.watchedItems[path]:
+            oldCnt = itm.childCount()
+            
+            qdir = QDir(itm.dirName())
+            
+            entryInfoList = qdir.entryInfoList(
+                QDir.Filters(QDir.AllEntries | QDir.NoDotAndDotDot))
+            
+            # step 1: check for new entries
+            children = itm.children()
+            for f in entryInfoList:
+                fpath = unicode(f.absoluteFilePath())
+                childFound = False
+                for child in children:
+                    if child.name() == fpath:
+                        childFound = True
+                        children.remove(child)
+                        break
+                if childFound:
+                    continue
+                
+                cnt = itm.childCount()
+                self.beginInsertRows(self.createIndex(itm.row(), 0, itm),
+                    cnt, cnt)
+                if f.isDir():
+                    node = BrowserDirectoryItem(itm,
+                        unicode(Utilities.toNativeSeparators(f.absoluteFilePath())), 
+                        False)
+                else:
+                    node = BrowserFileItem(itm,
+                        unicode(Utilities.toNativeSeparators(f.absoluteFilePath())))
+                self._addItem(node, itm)
+                self.endInsertRows()
+            
+            # step 2: check for removed entries
+            if len(entryInfoList) != itm.childCount():
+                for row in range(oldCnt - 1, -1, -1):
+                    child = itm.child(row)
+                    entryFound = False
+                    for f in entryInfoList:
+                        if unicode(f.absoluteFilePath()) == child.name():
+                            entryFound = True
+                            entryInfoList.remove(f)
+                            break
+                    if entryFound:
+                        continue
+                    
+                    self._removeWatchedItem(child)
+                    self.beginRemoveRows(self.createIndex(itm.row(), 0, itm),
+                        row, row)
+                    itm.removeChild(child)
+                    self.endRemoveRows()
+    
     def __populateModel(self):
         """
         Private method to populate the browser model.
@@ -246,19 +355,21 @@ class BrowserModel(QAbstractItemModel):
                     d.absoluteFilePath()))
         
         for d in self.toplevelDirs:
-            self._addItem(BrowserDirectoryItem(self.rootItem, d), self.rootItem)
+            itm = BrowserDirectoryItem(self.rootItem, d)
+            self._addItem(itm, self.rootItem)
     
     def programChange(self, dirname):
         """
         Public method to change the entry for the directory of file being debugged.
         
-        @param dirname name of the directory containg the file (string or QString)
+        @param dirname name of the directory containing the file (string or QString)
         """
         if self.progDir:
             if dirname == self.progDir.dirName():
                 return
             
             # remove old entry
+            self._removeWatchedItem(self.progDir)
             self.beginRemoveRows(QModelIndex(), self.progDir.row(), self.progDir.row())
             self.rootItem.removeChild(self.progDir)
             self.endRemoveRows()
@@ -294,6 +405,7 @@ class BrowserModel(QAbstractItemModel):
         self.endRemoveRows()
         
         self.toplevelDirs.removeAll(item.dirName())
+        self._removeWatchedItem(item)
     
     def saveToplevelDirs(self):
         """
@@ -355,6 +467,8 @@ class BrowserModel(QAbstractItemModel):
         @param parentItem reference to the directory item to be populated
         @param repopulate flag indicating a repopulation (boolean)
         """
+        self._addWatchedItem(parentItem)
+        
         qdir = QDir(parentItem.dirName())
         
         entryInfoList = \
@@ -369,6 +483,12 @@ class BrowserModel(QAbstractItemModel):
                         unicode(Utilities.toNativeSeparators(f.absoluteFilePath())), 
                         False)
                 else:
+                    fileFilters = Preferences.getUI("BrowsersFileFilters").split(";")
+                    if not fileFilters.isEmpty():
+                        fn = f.fileName()
+                        if any([fnmatch.fnmatch(fn, unicode(ff).strip()) \
+                                for ff in fileFilters]):
+                            continue
                     node = BrowserFileItem(parentItem,
                         unicode(Utilities.toNativeSeparators(f.absoluteFilePath())))
                 self._addItem(node, parentItem)
@@ -486,7 +606,8 @@ class BrowserModel(QAbstractItemModel):
         if len(cl.globals):
             node = BrowserClassAttributesItem(\
                 parentItem, cl.globals, 
-                QApplication.translate("BrowserModel", "Attributes (global)"))
+                QApplication.translate("BrowserModel", "Class Attributes"),
+                True)
             if repopulate:
                 self.addItem(node, 
                     self.createIndex(parentItem.row(), 0, parentItem))
@@ -533,6 +654,7 @@ class BrowserModel(QAbstractItemModel):
         @param parentItem reference to the class attributes item to be populated
         @param repopulate flag indicating a repopulation (boolean)
         """
+        classAttributes = parentItem.isClassAttributes()
         attributes = parentItem.attributes()
         if not attributes:
             return
@@ -543,7 +665,8 @@ class BrowserModel(QAbstractItemModel):
                 self.beginInsertRows(self.createIndex(parentItem.row(), 0, parentItem),
                     0, len(keys) - 1)
             for key in keys:
-                node = BrowserClassAttributeItem(parentItem, attributes[key])
+                node = BrowserClassAttributeItem(parentItem, attributes[key],
+                                                 classAttributes)
                 self._addItem(node, parentItem)
             if repopulate:
                 self.endInsertRows()
@@ -567,6 +690,7 @@ class BrowserItem(object):
         self.icon = UI.PixmapCache.getIcon("empty.png")
         self._populated = True
         self._lazyPopulation = False
+        self.symlink = False
     
     def appendChild(self, child):
         """
@@ -705,6 +829,14 @@ class BrowserItem(object):
             return self.itemData[column] < other.itemData[column]
         except IndexError:
             return False
+    
+    def isSymlink(self):
+        """
+        Public method to check, if the items is a symbolic link.
+        
+        @return flag indicating a symbolic link (boolean)
+        """
+        return self.symlink
 
 class BrowserDirectoryItem(BrowserItem):
     """
@@ -728,7 +860,11 @@ class BrowserDirectoryItem(BrowserItem):
         BrowserItem.__init__(self, parent, dn)
         
         self.type_ = BrowserItemDirectory
-        self.icon = UI.PixmapCache.getIcon("dirClosed.png")
+        if os.path.lexists(self._dirName) and os.path.islink(self._dirName):
+            self.symlink = True
+            self.icon = UI.PixmapCache.getSymlinkIcon("dirClosed.png")
+        else:
+            self.icon = UI.PixmapCache.getIcon("dirClosed.png")
         self._populated = False
         self._lazyPopulation = True
     
@@ -755,6 +891,14 @@ class BrowserDirectoryItem(BrowserItem):
         """
         return self._dirName
     
+    def name(self):
+        """
+        Public method to return the name of the item.
+        
+        @return name of the item (string)
+        """
+        return self._dirName
+    
     def lessThan(self, other, column, order):
         """
         Public method to check, if the item is less than the other one.
@@ -813,50 +957,57 @@ class BrowserFileItem(BrowserItem):
         
         self._moduleName = ''
         
+        pixName = ""
         if self.isPythonFile():
             if self.fileext == '.py':
-                self.icon = UI.PixmapCache.getIcon("filePython.png")
+                pixName = "filePython.png"
             else:
-                self.icon = UI.PixmapCache.getIcon("filePython2.png")
+                pixName = "filePython2.png"
             self._populated = False
             self._lazyPopulation = True
             self._moduleName = os.path.basename(finfo)
         elif self.isPython3File():
-            self.icon = UI.PixmapCache.getIcon("filePython.png")
+            pixName = "filePython.png"
             self._populated = False
             self._lazyPopulation = True
             self._moduleName = os.path.basename(finfo)
         elif self.isRubyFile():
-            self.icon = UI.PixmapCache.getIcon("fileRuby.png")
+            "fileRuby.png"
             self._populated = False
             self._lazyPopulation = True
             self._moduleName = os.path.basename(finfo)
         elif self.isDesignerFile() or self.isDesignerHeaderFile():
-            self.icon = UI.PixmapCache.getIcon("fileDesigner.png")
+            pixName = "fileDesigner.png"
         elif self.isLinguistFile():
             if self.fileext == '.ts':
-                self.icon = UI.PixmapCache.getIcon("fileLinguist.png")
+                pixName = "fileLinguist.png"
             else:
-                self.icon = UI.PixmapCache.getIcon("fileLinguist2.png")
+                pixName = "fileLinguist2.png"
         elif self.isResourcesFile():
-            self.icon = UI.PixmapCache.getIcon("fileResource.png")
+            pixName = "fileResource.png"
         elif self.isProjectFile():
-            self.icon = UI.PixmapCache.getIcon("fileProject.png")
+            pixName = "fileProject.png"
         elif self.isMultiProjectFile():
-            self.icon = UI.PixmapCache.getIcon("fileMultiProject.png")
+            pixName = "fileMultiProject.png"
         elif self.isIdlFile():
-            self.icon = UI.PixmapCache.getIcon("fileIDL.png")
+            pixName = "fileIDL.png"
             self._populated = False
             self._lazyPopulation = True
             self._moduleName = os.path.basename(finfo)
         elif self.isPixmapFile():
-            self.icon = UI.PixmapCache.getIcon("filePixmap.png")
+            pixName = "filePixmap.png"
         elif self.isSvgFile():
-            self.icon = UI.PixmapCache.getIcon("fileSvg.png")
+            pixName = "fileSvg.png"
         elif self.isDFile():
-            self.icon = UI.PixmapCache.getIcon("fileD.png")
+            pixName = "fileD.png"
         else:
-            self.icon = UI.PixmapCache.getIcon("fileMisc.png")
+            pixName = "fileMisc.png"
+        
+        if os.path.lexists(self._filename) and os.path.islink(self._filename):
+            self.symlink = True
+            self.icon = UI.PixmapCache.getSymlinkIcon(pixName)
+        else:
+            self.icon = UI.PixmapCache.getIcon(pixName)
     
     def setName(self, finfo, full = True):
         """
@@ -888,6 +1039,14 @@ class BrowserFileItem(BrowserItem):
         """
         return self.fileext
     
+    def name(self):
+        """
+        Public method to return the name of the item.
+        
+        @return name of the item (string)
+        """
+        return self._filename
+    
     def dirName(self):
         """
         Public method returning the directory name.
@@ -1170,7 +1329,13 @@ class BrowserMethodItem(BrowserItem):
         self.name = name
         self._functionObject = fn
         self._filename = filename
-        if self._functionObject.isPrivate():
+        if self._functionObject.modifier == \
+           Utilities.ClassBrowsers.ClbrBaseClasses.Function.Static:
+            self.icon = UI.PixmapCache.getIcon("method_static.png")
+        elif self._functionObject.modifier == \
+           Utilities.ClassBrowsers.ClbrBaseClasses.Function.Class:
+            self.icon = UI.PixmapCache.getIcon("method_class.png")
+        elif self._functionObject.isPrivate():
             self.icon = UI.PixmapCache.getIcon("method_private.png")
         elif self._functionObject.isProtected():
             self.icon = UI.PixmapCache.getIcon("method_protected.png")
@@ -1247,13 +1412,14 @@ class BrowserClassAttributesItem(BrowserItem):
     """
     Class implementing the data structure for browser class attributes items.
     """
-    def __init__(self, parent, attributes, text):
+    def __init__(self, parent, attributes, text, isClass=False):
         """
         Constructor
         
         @param parent parent item
         @param attributes list of attributes
         @param text text to be shown by this item (QString)
+        @param isClass flag indicating class attributes (boolean)
         """
         BrowserItem.__init__(self, parent, text)
         
@@ -1261,7 +1427,11 @@ class BrowserClassAttributesItem(BrowserItem):
         self._attributes = attributes.copy()
         self._populated = False
         self._lazyPopulation = True
-        self.icon = UI.PixmapCache.getIcon("attributes.png")
+        if isClass:
+            self.icon = UI.PixmapCache.getIcon("attributes_class.png")
+        else:
+            self.icon = UI.PixmapCache.getIcon("attributes.png")
+        self.__isClass = isClass
     
     def attributes(self):
         """
@@ -1271,6 +1441,14 @@ class BrowserClassAttributesItem(BrowserItem):
         """
         return self._attributes
     
+    def isClassAttributes(self):
+        """
+        Public method returning the attributes type.
+        
+        @return flag indicating class attributes (boolean)
+        """
+        return self.__isClass
+    
     def lessThan(self, other, column, order):
         """
         Public method to check, if the item is less than the other one.
@@ -1292,19 +1470,22 @@ class BrowserClassAttributeItem(BrowserItem):
     """
     Class implementing the data structure for browser class attribute items.
     """
-    def __init__(self, parent, attribute):
+    def __init__(self, parent, attribute, isClass=False):
         """
         Constructor
         
         @param parent parent item
         @param attribute reference to the attribute object
+        @param isClass flag indicating a class attribute (boolean)
         """
         BrowserItem.__init__(self, parent, attribute.name)
         
         self.type_ = BrowserItemAttribute
         self._attributeObject = attribute
         self.__public = attribute.isPublic()
-        if attribute.isPrivate():
+        if isClass:
+            self.icon = UI.PixmapCache.getIcon("attribute_class.png")
+        elif attribute.isPrivate():
             self.icon = UI.PixmapCache.getIcon("attribute_private.png")
         elif attribute.isProtected():
             self.icon = UI.PixmapCache.getIcon("attribute_protected.png")
