@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2003 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2003 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -256,7 +256,7 @@ class EmailDialog(QDialog, Ui_EmailDialog):
                     res = KQMessageBox.critical(self,
                         self.trUtf8("Send bug report"),
                         self.trUtf8("""<p>Authentication failed.<br>Reason: %1</p>""")
-                            .arg(unicode(e)),
+                            .arg(str(e)),
                         QMessageBox.StandardButtons(\
                             QMessageBox.Abort | \
                             QMessageBox.Retry),
@@ -277,7 +277,7 @@ class EmailDialog(QDialog, Ui_EmailDialog):
             res = KQMessageBox.critical(self,
                 self.trUtf8("Send bug report"),
                 self.trUtf8("""<p>Message could not be sent.<br>Reason: %1</p>""")
-                    .arg(unicode(e)),
+                    .arg(str(e)),
                 QMessageBox.StandardButtons(\
                     QMessageBox.Abort | \
                     QMessageBox.Retry),
