@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -39,6 +39,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
     startRole   = Qt.UserRole + 2
     endRole     = Qt.UserRole + 3
     replaceRole = Qt.UserRole + 4
+    md5Role     = Qt.UserRole + 5
     
     def __init__(self, project, replaceMode = False, parent=None):
         """
@@ -72,8 +73,16 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         
         self.findProgressLabel.setMaximumWidth(550)
         
-        self.searchHistory = QStringList()
-        self.replaceHistory = QStringList()
+        self.searchHistory = Preferences.Prefs.settings.value(
+            "FindFileDialog/SearchHistory").toStringList()
+        self.replaceHistory = Preferences.Prefs.settings.value(
+            "FindFileDialog/ReplaceHistory").toStringList()
+        self.dirHistory = Preferences.Prefs.settings.value(
+            "FindFileDialog/DirectoryHistory").toStringList()
+        self.findtextCombo.addItems(self.searchHistory)
+        self.replacetextCombo.addItems(self.replaceHistory)
+        self.dirCombo.addItems(self.dirHistory)
+        
         self.project = project
         
         self.findList.headerItem().setText(self.findList.columnCount(), "")
@@ -108,7 +117,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         self.connect(self, SIGNAL("customContextMenuRequested(const QPoint &)"), 
                      self.__contextMenuRequested)
         
-    def __createItem(self, file, line, text, start, end, replTxt = ""):
+    def __createItem(self, file, line, text, start, end, replTxt = "", md5 = ""):
         """
         Private method to create an entry in the file list.
         
@@ -117,7 +126,8 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         @param text text found (string or QString)
         @param start start position of match (integer)
         @param end end position of match (integer)
-        @param replTxt text with replacements applied (string or QString
+        @param replTxt text with replacements applied (string or QString)
+        @param md5 MD5 hash of the file (string or QString)
         """
         if self.__lastFileItem is None:
             # It's a new file
@@ -129,6 +139,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
                     Qt.ItemFlags(Qt.ItemIsUserCheckable | Qt.ItemIsTristate))
                 # Qt bug: 
                 # item is not user checkable if setFirstColumnSpanned is True (< 4.5.0)
+            self.__lastFileItem.setData(0, self.md5Role, QVariant(md5))
         
         itm = QTreeWidgetItem(self.__lastFileItem, [' %5d ' % line, text])
         itm.setTextAlignment(0,  Qt.AlignRight)
@@ -179,9 +190,9 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         """
         self.__enableFindButton()
         
-    def on_dirEdit_textChanged(self, text):
+    def on_dirCombo_editTextChanged(self, text):
         """
-        Private slot to handle the textChanged signal of the directory edit.
+        Private slot to handle the textChanged signal of the directory combo box.
         
         @param text (ignored)
         """
@@ -224,8 +235,9 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         if self.findtextCombo.currentText().isEmpty() or \
            (self.__replaceMode and self.replacetextCombo.currentText().isEmpty()) or \
            (self.dirButton.isChecked() and \
-            (self.dirEdit.text().isEmpty() or \
-             not os.path.exists(os.path.abspath(unicode(self.dirEdit.text()))))) or \
+            (self.dirCombo.currentText().isEmpty() or \
+             not os.path.exists(
+                os.path.abspath(unicode(self.dirCombo.currentText()))))) or \
            (self.filterCheckBox.isChecked() and self.filterEdit.text().isEmpty()):
             self.findButton.setEnabled(False)
             self.buttonBox.button(QDialogButtonBox.Close).setDefault(True)
@@ -279,8 +291,9 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         
         if self.projectButton.isChecked():
             if self.filterCheckBox.isChecked():
-                files = [file.replace(self.project.ppath+os.sep, "") \
-                         for file in self.__getFileList(self.project.ppath, filterRe)]
+                files = [self.project.getRelativePath(file) \
+                         for file in \
+                         self.__getFileList(self.project.getProjectPath(), filterRe)]
             else:
                 files = []
                 if self.sourcesCheckBox.isChecked():
@@ -308,8 +321,8 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
                     filters.append(self.filterResources)
                 filterString = "|".join(filters)
                 filterRe = re.compile(filterString)
-            files = self.__getFileList(os.path.abspath(unicode(self.dirEdit.text())), 
-                                       filterRe)
+            files = self.__getFileList(
+                os.path.abspath(unicode(self.dirCombo.currentText())), filterRe)
         elif self.openFilesButton.isChecked():
             files = e4App().getObject("ViewManager").getOpenFilenames()
         
@@ -338,7 +351,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
             QMessageBox.critical(None,
                 self.trUtf8("Invalid search expression"),
                 self.trUtf8("""<p>The search expression is not valid.</p>"""
-                            """<p>Error: %1</p>""").arg(unicode(why)))
+                            """<p>Error: %1</p>""").arg(str(why)))
             self.stopButton.setEnabled(False)
             self.findButton.setEnabled(True)
             self.findButton.setDefault(True)
@@ -349,12 +362,26 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         self.searchHistory.prepend(ct)
         self.findtextCombo.clear()
         self.findtextCombo.addItems(self.searchHistory)
+        Preferences.Prefs.settings.setValue("FindFileDialog/SearchHistory", 
+                                            self.searchHistory[:30])
+        
         if self.__replaceMode:
             replTxt = unicode(self.replacetextCombo.currentText())
             self.replaceHistory.removeAll(replTxt)
             self.replaceHistory.prepend(replTxt)
             self.replacetextCombo.clear()
             self.replacetextCombo.addItems(self.replaceHistory)
+            Preferences.Prefs.settings.setValue("FindFileDialog/ReplaceHistory", 
+                                                self.replaceHistory[:30])
+        
+        if self.dirButton.isChecked():
+            searchDir = self.dirCombo.currentText()
+            self.dirHistory.removeAll(searchDir)
+            self.dirHistory.insert(0, searchDir)
+            self.dirCombo.clear()
+            self.dirCombo.addItems(self.dirHistory)
+            Preferences.Prefs.settings.setValue("FindFileDialog/DirectoryHistory", 
+                                                self.dirHistory[:30])
         
         # now go through all the files
         self.__populating = True
@@ -375,7 +402,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
             # read the file and split it into textlines
             try:
                 f = open(fn, 'rb')
-                text, encoding = Utilities.decode(f.read())
+                text, encoding, hash = Utilities.decodeWithHash(f.read())
                 lines = text.splitlines(True)
                 f.close()
             except IOError:
@@ -406,7 +433,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
                             rline = "%s ..." % line[:1024]
                         line = "- %s\n+ %s" % (
                             line, self.__stripEol(rline))
-                    self.__createItem(file, count, line, start, end, rline)
+                    self.__createItem(file, count, line, start, end, rline, hash)
                     
                     if self.feelLikeCheckBox.isChecked():
                         fn = os.path.join(self.project.ppath, unicode(file))
@@ -479,11 +506,11 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         directory = KQFileDialog.getExistingDirectory(\
             self,
             self.trUtf8("Select directory"),
-            self.dirEdit.text(),
+            self.dirCombo.currentText(),
             QFileDialog.Options(QFileDialog.ShowDirsOnly))
             
         if not directory.isEmpty():
-            self.dirEdit.setText(Utilities.toNativeSeparators(directory))
+            self.dirCombo.setEditText(Utilities.toNativeSeparators(directory))
         
     def __getFileList(self, path, filterRe):
         """
@@ -509,7 +536,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         @param searchDir name of the directory to search in (string or QString)
         """
         self.dirButton.setChecked(True)
-        self.dirEdit.setText(Utilities.toNativeSeparators(searchDir))
+        self.dirCombo.setEditText(Utilities.toNativeSeparators(searchDir))
         
     @pyqtSignature("")
     def on_replaceButton_clicked(self):
@@ -524,6 +551,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
             itm = self.findList.topLevelItem(index)
             if itm.checkState(0) in [Qt.PartiallyChecked, Qt.Checked]:
                 file = unicode(itm.text(0))
+                origHash = itm.data(0, self.md5Role).toString()
                 
                 self.findProgressLabel.setPath(file)
                 
@@ -535,7 +563,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
                 # read the file and split it into textlines
                 try:
                     f = open(fn, 'rb')
-                    text, encoding = Utilities.decode(f.read())
+                    text, encoding, hash = Utilities.decodeWithHash(f.read())
                     lines = text.splitlines(True)
                     f.close()
                 except IOError, err:
@@ -543,7 +571,21 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
                         self.trUtf8("Replace in Files"),
                         self.trUtf8("""<p>Could not read the file <b>%1</b>."""
                                     """ Skipping it.</p><p>Reason: %2</p>""")\
-                            .arg(fn).arg(unicode(err))
+                            .arg(fn).arg(str(err))
+                    )
+                    progress += 1
+                    self.findProgress.setValue(progress)
+                    continue
+                
+                # Check the original and the current hash. Skip the file,
+                # if hashes are different.
+                if origHash != hash:
+                    KQMessageBox.critical(self,
+                        self.trUtf8("Replace in Files"),
+                        self.trUtf8("""<p>The current and the original hash of the"""
+                                    """ file <b>%1</b> are different. Skipping it."""
+                                    """</p><p>Hash 1: %2</p><p>Hash 2: %3</p>""")\
+                            .arg(fn).arg(origHash).arg(hash)
                     )
                     progress += 1
                     self.findProgress.setValue(progress)
@@ -569,7 +611,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
                         self.trUtf8("Replace in Files"),
                         self.trUtf8("""<p>Could not save the file <b>%1</b>."""
                                     """ Skipping it.</p><p>Reason: %2</p>""")\
-                            .arg(fn).arg(unicode(err))
+                            .arg(fn).arg(str(err))
                     )
             
             progress += 1
