@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2006 - 2011 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2006 - 2012 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -12,6 +12,8 @@ import os
 from PyQt4.QtCore import QThread, QMutex, QMutexLocker, QWaitCondition, \
     QString, QStringList, SIGNAL
 
+from KdeQt.KQApplication import e4App
+
 class VcsStatusMonitorThread(QThread):
     """
     Class implementing the VCS status monitor thread base class.
@@ -159,7 +161,8 @@ class VcsStatusMonitorThread(QThread):
         
         @param name name of the entry to be cleared (QString or string)
         """
-        key = unicode(name).replace(unicode(self.projectDir) + os.sep, '')
+        project = e4App().getObject("Project")
+        key = project.getRelativePath(unicode(name))
         try:
             del self.reportedStates[key]
         except KeyError:
